@@ -1,5 +1,5 @@
 /** \file 
-The V2 API Funtions.
+The V2 API Functions.
 
 Copyright 1996, University Corporation for Atmospheric Research
 See \ref copyright file for copying and redistribution conditions.
@@ -7,7 +7,7 @@ See \ref copyright file for copying and redistribution conditions.
 
 #ifndef NO_NETCDF_2
 
-#include <config.h>
+#include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -73,8 +73,9 @@ static void* nvmalloc(off_t size) {
   type *const name = (type*) nvmalloc((ndims) * sizeof(type))
 
 
-//  ALLOC_ONSTACK(name, type, ndims)		
-
+#if 0
+  ALLOC_ONSTACK(name, type, ndims)		
+#endif
 
 # define A_FREE(name) \
 	FREE_ONSTACK(name)
@@ -104,7 +105,7 @@ typedef signed char schar;
  * the record variable ids, if the array parameter is non-null.
  */
 static int
-numrecvars(int ncid, int *nrecvarsp, int *recvarids)
+numrecvars(int ncid, int* nrecvarsp, int *recvarids)
 {
     int status = NC_NOERR;
     int nvars = 0;
@@ -158,7 +159,7 @@ ncrecsize(int ncid, int varid, size_t *recsizep)
     int ndims;
     int dimids[MAX_NC_DIMS];
     int id;
-    size_t size;
+    int size;
 
     *recsizep = 0;
     status = nc_inq_unlimdim(ncid, &recdimid); 
@@ -182,9 +183,9 @@ ncrecsize(int ncid, int varid, size_t *recsizep)
 	status = nc_inq_dimlen(ncid, dimids[id], &len);
 	if(status != NC_NOERR)
 		return status;
-	size *= len;
+	size *= (int)len;
     }
-    *recsizep = size;
+    *recsizep = (size_t)size;
     return NC_NOERR;
 }
 
@@ -247,7 +248,6 @@ nc_inq_rec(
     if(status != NC_NOERR)
 	return status;
 
-    *nrecvarsp = 0;
     if (recdimid == -1)
 	return NC_NOERR;
     
@@ -256,7 +256,8 @@ nc_inq_rec(
 	return status;
 
     if (nrecvarsp != NULL)
-	*nrecvarsp = nrvars;
+	*nrecvarsp = (size_t)nrvars;
+
     if (recvarids != NULL)
 	for (varid = 0; varid < nrvars; varid++)
 	    recvarids[varid] = rvarids[varid];
@@ -864,23 +865,37 @@ ncvarputg(
     const void* value
 )
 {
+	int ndims = 0;
 	if(map == NULL)
 		return ncvarputs(ncid, varid, start, count, stride, value);
 	/* else */
 	{
-	NDIMS_DECL
+	ptrdiff_t *imp=NULL;
+	if (map != NULL) {
+		int ret = NC_NOERR;
+		/* make map[ndims-1] number of elements instead of bytes */
+		int i, el_size;
+		nc_type type;
+		ret = nc_inq_varndims(ncid, varid, &ndims);
+		if(ret) return ret;
+		ret = nc_inq_vartype(ncid, varid, &type);
+		if(ret) return ret;
+				el_size = nctypelen(type);
+		imp = (ptrdiff_t*) malloc(ndims * sizeof(ptrdiff_t));
+		for (i=0; i<ndims; i++) imp[i] = map[i] / el_size;
+	}
+
+	{
 	A_DECL(stp, size_t, ndims, start);
 	A_DECL(cntp, size_t, ndims, count);
 	A_DECL(strdp, ptrdiff_t, ndims, stride);
-	A_DECL(imp, ptrdiff_t, ndims, map);
 	A_INIT(stp, size_t, ndims, start);
 	A_INIT(cntp, size_t, ndims, count);
 	A_INIT(strdp, ptrdiff_t, ndims, stride);
-	A_INIT(imp, ptrdiff_t, ndims, map);
 	{
 	const int status = nc_put_varm(ncid, varid,
 			 stp, cntp, strdp, imp, value);
-	A_FREE(imp);
+	if (imp!=NULL) free(imp);
 	A_FREE(strdp);
 	A_FREE(cntp);
 	A_FREE(stp);
@@ -892,6 +907,7 @@ ncvarputg(
 	}
 	return 0;
 	}
+	}
 }
 
 
@@ -906,23 +922,37 @@ ncvargetg(
     void*	value
 )
 {
+	int ndims = 0;
 	if(map == NULL)
 		return ncvargets(ncid, varid, start, count, stride, value);
 	/* else */
 	{
-	NDIMS_DECL
+	ptrdiff_t *imp=NULL;
+	if (map != NULL) {
+		int ret = NC_NOERR;
+		/* make map[ndims-1] number of elements instead of bytes */
+		int i, el_size;
+		nc_type type;
+		ret = nc_inq_varndims(ncid, varid, &ndims);
+		if(ret) return ret;
+		ret = nc_inq_vartype(ncid, varid, &type);
+		if(ret) return ret;
+		el_size = nctypelen(type);
+		imp = (ptrdiff_t*) malloc(ndims * sizeof(ptrdiff_t));
+		for (i=0; i<ndims; i++) imp[i] = map[i] / el_size;
+	}
+
+	{
 	A_DECL(stp, size_t, ndims, start);
 	A_DECL(cntp, size_t, ndims, count);
 	A_DECL(strdp, ptrdiff_t, ndims, stride);
-	A_DECL(imp, ptrdiff_t, ndims, map);
 	A_INIT(stp, size_t, ndims, start);
 	A_INIT(cntp, size_t, ndims, count);
 	A_INIT(strdp, ptrdiff_t, ndims, stride);
-	A_INIT(imp, ptrdiff_t, ndims, map);
 	{
 	const int status = nc_get_varm(ncid, varid,
 			stp, cntp, strdp, imp, value);
-	A_FREE(imp);
+	if (imp!=NULL) free(imp);
 	A_FREE(strdp);
 	A_FREE(cntp);
 	A_FREE(stp);
@@ -934,6 +964,7 @@ ncvargetg(
 	}
 	return 0;
 	}
+	}
 }
 
 
@@ -1120,11 +1151,18 @@ ncrecinq(
 )
 {
 	size_t nrv = 0;
-	size_t rs[NC_MAX_VARS]; /* TODO */
-	const int status = nc_inq_rec(ncid, &nrv, recvarids, rs);
+	size_t *rs = NULL;
+	int status = NC_NOERR;
+
+	rs = (size_t*)malloc(sizeof(size_t)*NC_MAX_VARS);
+	if(rs == NULL)
+	    return NC_ENOMEM;
+
+	status = nc_inq_rec(ncid, &nrv, recvarids, rs);
 	if(status != NC_NOERR)
 	{
 		nc_advise("ncrecinq", status, "ncid %d", ncid);
+		if(rs != NULL) free(rs);
 		return -1;
 	}
 
@@ -1140,6 +1178,8 @@ ncrecinq(
 		}
 	}
 
+	if(rs != NULL) free(rs);
+
 	return (int) nrv;
 }
 
