@@ -5,30 +5,44 @@
 /* $Id: posixio.c,v 1.89 2010/05/22 21:59:08 dmh Exp $ */
 
 /* For MinGW Build */
+
+#if HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <stdio.h>
+#include <assert.h>
+#include <stdlib.h>
+#include <errno.h>
+#include <string.h>
+
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+
+/* Windows platforms, including MinGW, Cygwin, Visual Studio */
 #if defined(_WIN32) || defined(_WIN64)
 #include <windows.h>
 #include <winbase.h>
 #include <io.h>
 #endif
 
-#include <config.h>
-#include <assert.h>
-#include <stdlib.h>
-#include <errno.h>
-#ifndef ENOERR
-#define ENOERR 0
-#endif
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <string.h>
-#ifdef _MSC_VER /* Microsoft Compilers */
-#include <io.h>
-#else
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+
+#ifndef NC_NOERR
+#define NC_NOERR 0
+#endif
+
 #ifndef HAVE_SSIZE_T
-#define ssize_t int
+typedef int ssize_t;
 #endif
 
 #ifndef SEEK_SET
@@ -94,6 +108,41 @@ static int ncio_spx_close(ncio *nciop, int doUnlink);
 #ifndef POSIXIO_DEFAULT_PAGESIZE
 #define POSIXIO_DEFAULT_PAGESIZE 4096
 #endif
+
+/*! Cross-platform file length.
+ *
+ * Some versions of Visual Studio are throwing errno 132
+ * when fstat is used on large files.  This function is
+ * an attempt to get around that.
+ *
+ * @par fd File Descriptor.
+ * @return -1 on error, length of file (in bytes) otherwise.
+ */
+static off_t nc_get_filelen(const int fd) {
+
+  off_t flen;
+
+#ifdef HAVE_FILE_LENGTH_I64
+  __int64 file_len = 0;
+  if ((file_len = _filelengthi64(fd)) < 0) {
+    return file_len;
+  }
+  flen = (off_t)file_len;
+
+#else
+  int res = 0;
+  struct stat sb;
+  if((res = fstat(fd,&sb)) <0)
+    return res;
+
+  flen = sb.st_size;
+#endif
+
+  return flen;
+
+}
+
+
 /*
  * What is the system pagesize?
  */
@@ -129,7 +178,8 @@ pagesize(void)
 static size_t
 blksize(int fd)
 {
-#if defined(HAVE_ST_BLKSIZE)
+#ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
+#ifdef HAVE_SYS_STAT_H
 	struct stat sb;
 	if (fstat(fd, &sb) > -1)
 	{
@@ -138,6 +188,7 @@ blksize(int fd)
 		return 8192;
 	}
 	/* else, silent in the face of error */
+#endif
 #endif
 	return (size_t) 2 * pagesize();
 }
@@ -154,7 +205,7 @@ fgrow(const int fd, const off_t len)
 	if (fstat(fd, &sb) < 0)
 		return errno;
 	if (len < sb.st_size)
-		return ENOERR;
+		return NC_NOERR;
 	{
 	    const long dumb = 0;
 	    /* we don't use ftruncate() due to problem with FAT32 file systems */
@@ -169,7 +220,7 @@ fgrow(const int fd, const off_t len)
 	    if (lseek(fd, pos, SEEK_SET) < 0)
 		return errno;
 	}
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -181,13 +232,23 @@ fgrow(const int fd, const off_t len)
 static int
 fgrow2(const int fd, const off_t len)
 {
-	struct stat sb;
-	if (fstat(fd, &sb) < 0)
-		return errno;
-	if (len <= sb.st_size)
-		return ENOERR;
-	{
-	    const char dumb = 0;
+
+
+  /* There is a problem with fstat on Windows based systems
+     which manifests (so far) when Config RELEASE is built.
+     Use _filelengthi64 isntead.
+
+     See https://github.com/Unidata/netcdf-c/issues/188
+
+  */
+
+
+  off_t file_len = nc_get_filelen(fd);
+  if(file_len < 0) return errno;
+  if(len <= file_len)
+    return NC_NOERR;
+  {
+    const char dumb = 0;
 	    /* we don't use ftruncate() due to problem with FAT32 file systems */
 	    /* cache current position */
 	    const off_t pos = lseek(fd, 0, SEEK_CUR);
@@ -200,7 +261,7 @@ fgrow2(const int fd, const off_t len)
 	    if (lseek(fd, pos, SEEK_SET) < 0)
 		return errno;
 	}
-	return ENOERR;
+	return NC_NOERR;
 }
 /* End OS */
 /* Begin px */
@@ -218,7 +279,7 @@ fgrow2(const int fd, const off_t len)
    posp - pointer to current position in file, updated after write.
 */
 static int
-px_pgout(ncio *const nciop, 
+px_pgout(ncio *const nciop,
 	off_t const offset,  const size_t extent,
 	void *const vp, off_t *posp)
 {
@@ -249,24 +310,26 @@ px_pgout(ncio *const nciop,
 	while((partial = write(nciop->fd, nvp, nextent)) != -1) {
 	    if(partial == nextent)
 		break;
-	    nvp += partial;		
+	    nvp += partial;
 	    nextent -= partial;
 	}
 	if(partial == -1)
 	    return errno;
 	*posp += extent;
 
-	return ENOERR;
+	return NC_NOERR;
 }
 
-/* Read in a page of data. 
+/*! Read in a page of data.
 
-   nciop - a pointer to the ncio struct for this file.
-   offset - byte offset in file where read starts.
-   extent - the size of the page that will be read.
-   vp - a pointer to where the data will end up.
-   nreadp - returned number of bytes actually read (may be less than extent).
-   posp - pointer to current position in file, updated after read.
+  @param[in] nciop  A pointer to the ncio struct for this file.
+  @param[in] offset The byte offset in file where read starts.
+  @param[in] extent The size of the page that will be read.
+  @param[in] vp     A pointer to where the data will end up.
+
+  @param[in,out] nreadp Returned number of bytes actually read (may be less than extent).
+  @param[in,out] posp The pointer to current position in file, updated after read.
+  @return Return 0 on success, otherwise an error code.
 */
 static int
 px_pgin(ncio *const nciop,
@@ -275,13 +338,23 @@ px_pgin(ncio *const nciop,
 {
 	int status;
 	ssize_t nread;
-
+    size_t read_count = 0;
+    ssize_t bytes_xfered = 0;
+    void *p = vp;
 #ifdef X_ALIGN
 	assert(offset % X_ALIGN == 0);
 	assert(extent % X_ALIGN == 0);
 #endif
-
-	assert(*posp == OFF_NONE || *posp == lseek(nciop->fd, 0, SEEK_CUR));
+    /* *posp == OFF_NONE (-1) on first call. This
+       is problematic because lseek also returns -1
+       on error. Use errno instead. */
+    if(*posp != OFF_NONE && *posp != lseek(nciop->fd, 0, SEEK_CUR)) {
+      if(errno) {
+        status = errno;
+        printf("Error %d: %s\n",errno,strerror(errno));
+        return status;
+      }
+    }
 
 	if(*posp != offset)
 	{
@@ -294,19 +367,37 @@ px_pgin(ncio *const nciop,
 	}
 
 	errno = 0;
-	nread = read(nciop->fd, vp, extent);
-	if(nread != (ssize_t) extent)
-	{
-		status = errno;
-		if(nread == -1 || status != ENOERR)
-			return status;
-		/* else it's okay we read less than asked for */
-		(void) memset((char *)vp + nread, 0, (ssize_t)extent - nread);
-	}
-	*nreadp = nread;
+    /* Handle the case where the read is interrupted
+       by a signal (see NCF-337,
+       http://pubs.opengroup.org/onlinepubs/009695399/functions/read.html)
+
+       When this happens, nread will (should) be the bytes read, and
+       errno will be set to EINTR.  On older systems nread might be -1.
+       If this is the case, there's not a whole lot we can do about it
+       as we can't compute any offsets, so we will attempt to read again.
+       This *feels* like it could lead to an infinite loop, but it shouldn't
+       unless the read is being constantly interrupted by a signal, and is
+       on an older system which returns -1 instead of bytexs read.
+
+       The case where it's a short read is already handled by the function
+       (according to the comment below, at least). */
+    do {
+      nread = read(nciop->fd,vp,extent);
+    } while (nread == -1 && errno == EINTR);
+
+
+    if(nread != (ssize_t)extent) {
+      status = errno;
+      if( nread == -1 || (status != EINTR && status != NC_NOERR))
+        return status;
+      /* else it's okay we read less than asked for */
+      (void) memset((char *)vp + nread, 0, (ssize_t)extent - nread);
+    }
+
+    *nreadp = nread;
 	*posp += nread;
 
-	return ENOERR;
+	return NC_NOERR;
 }
 
 /* This struct is for POSIX systems, with NC_SHARE not in effect. If
@@ -328,7 +419,7 @@ typedef struct ncio_px {
 	size_t blksz;
 	off_t pos;
 	/* buffer */
-	off_t	bf_offset; 
+	off_t	bf_offset;
 	size_t	bf_extent;
 	size_t	bf_cnt;
 	void	*bf_base;
@@ -352,7 +443,7 @@ typedef struct ncio_px {
    offset - file offset for beginning of to region to be
    released.
 
-   rflags - only RGN_MODIFIED is relevent to this function, others ignored
+   rflags - only RGN_MODIFIED is relevant to this function, others ignored
 */
 static int
 px_rel(ncio_px *const pxp, off_t offset, int rflags)
@@ -368,7 +459,7 @@ px_rel(ncio_px *const pxp, off_t offset, int rflags)
 	}
 	pxp->bf_refcount--;
 
-	return ENOERR;
+	return NC_NOERR;
 }
 
 /* This function indicates the file region starting at offset may be
@@ -385,7 +476,7 @@ px_rel(ncio_px *const pxp, off_t offset, int rflags)
    nciop - pointer to ncio struct.
    offset - num bytes from beginning of buffer to region to be
    released.
-   rflags - only RGN_MODIFIED is relevent to this function, others ignored
+   rflags - only RGN_MODIFIED is relevant to this function, others ignored
 */
 static int
 ncio_px_rel(ncio *const nciop, off_t offset, int rflags)
@@ -401,7 +492,7 @@ ncio_px_rel(ncio *const nciop, off_t offset, int rflags)
 /* POSIX get. This will "make a region available." Since we're using
    buffered IO, this means that if needed, we'll fetch a new page from
    the file, otherwise, just return a pointer to what's in memory
-   already. 
+   already.
 
    nciop - pointer to ncio struct, containing file info.
    pxp - pointer to ncio_px struct, which contains special metadate
@@ -409,9 +500,9 @@ ncio_px_rel(ncio *const nciop, off_t offset, int rflags)
    offset - start byte of region to get.
    extent - how many bytes to read.
    rflags - One of the RGN_* flags defined in ncio.h.
-   vpp - pointer to pointer that will recieve data.
+   vpp - pointer to pointer that will receive data.
 
-   NOTES: 
+   NOTES:
 
    * For blkoffset round offset down to the nearest pxp->blksz. This
    provides the offset (in bytes) to the beginning of the block that
@@ -431,7 +522,7 @@ ncio_px_rel(ncio *const nciop, off_t offset, int rflags)
 
    * If this is called on a newly opened file, pxp->bf_offset will be
    OFF_NONE and we'll jump to label pgin to immediately read in a
-   page. 
+   page.
 */
 static int
 px_get(ncio *const nciop, ncio_px *const pxp,
@@ -439,12 +530,12 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 		int rflags,
 		void **const vpp)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 
 	const off_t blkoffset = _RNDDOWN(offset, (off_t)pxp->blksz);
 	off_t diff = (size_t)(offset - blkoffset);
 	off_t blkextent = _RNDUP(diff + extent, pxp->blksz);
-	
+
 	assert(extent != 0);
 	assert(extent < X_INT_MAX); /* sanity check */
 	assert(offset >= 0); /* sanity check */
@@ -470,7 +561,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 	if(blkoffset == pxp->bf_offset)
 	{
 		/* hit */
- 		if(blkextent > pxp->bf_extent) 
+ 		if(blkextent > pxp->bf_extent)
 		{
 			/* page in upper */
 			void *const middle =
@@ -482,7 +573,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 				 middle,
 				 &pxp->bf_cnt,
 				 &pxp->pos);
-			if(status != ENOERR)
+			if(status != NC_NOERR)
 				return status;
 			pxp->bf_extent = 2 * pxp->blksz;
 			pxp->bf_cnt += pxp->blksz;
@@ -517,7 +608,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 					pxp->blksz,
 					pxp->bf_base,
 					&pxp->pos);
-				if(status != ENOERR)
+				if(status != NC_NOERR)
 					return status;
 			}
 			pxp->bf_cnt -= pxp->blksz;
@@ -536,7 +627,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 					pxp->blksz,
 					pxp->bf_base,
 					&pxp->pos);
-				if(status != ENOERR)
+				if(status != NC_NOERR)
 					return status;
 			}
 		}
@@ -554,7 +645,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 				 middle,
 				 &pxp->bf_cnt,
 				 &pxp->pos);
-			if(status != ENOERR)
+			if(status != NC_NOERR)
 				return status;
 			pxp->bf_extent = 2 * pxp->blksz;
 			pxp->bf_cnt += pxp->blksz;
@@ -582,7 +673,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 					pxp->bf_cnt - pxp->blksz,
 					middle,
 					&pxp->pos);
-				if(status != ENOERR)
+				if(status != NC_NOERR)
 					return status;
 			}
 			pxp->bf_cnt = pxp->blksz;
@@ -601,7 +692,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 			 pxp->bf_base,
 			 &pxp->bf_cnt,
 			 &pxp->pos);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			return status;
 		pxp->bf_offset = blkoffset;
 		if(upper_cnt != 0)
@@ -626,7 +717,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 			pxp->bf_cnt,
 			pxp->bf_base,
 			&pxp->pos);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			return status;
 		pxp->bf_rflags = 0;
 	}
@@ -638,7 +729,7 @@ pgin:
 		 pxp->bf_base,
 		 &pxp->bf_cnt,
 		 &pxp->pos);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 	 pxp->bf_offset = blkoffset;
 	 pxp->bf_extent = blkextent;
@@ -652,8 +743,12 @@ done:
 	pxp->bf_rflags |= rflags;
 	pxp->bf_refcount++;
 
-	*vpp = (char *)pxp->bf_base + diff;
-	return ENOERR;
+#ifndef __CHAR_UNSIGNED__
+    *vpp = (void *)((char *)pxp->bf_base + diff);
+#else
+    *vpp = (void *)((signed char*)pxp->bf_base + diff);
+#endif
+	return NC_NOERR;
 }
 
 /* Request that the region (offset, extent) be made available through
@@ -664,7 +759,7 @@ done:
    corresponding call to rel().
 
    For POSIX systems, without NC_SHARE. This function gets a page of
-   size extent? 
+   size extent?
 
    This is a wrapper for the function px_get, which does all the heavy
    lifting.
@@ -677,13 +772,13 @@ done:
    vpp - handle to point at data when it's been read.
 */
 static int
-ncio_px_get(ncio *const nciop, 
+ncio_px_get(ncio *const nciop,
 		off_t offset, size_t extent,
 		int rflags,
 		void **const vpp)
 {
 	ncio_px *const pxp = (ncio_px *)nciop->pvt;
-	
+
 	if(fIsSet(rflags, RGN_WRITE) && !fIsSet(nciop->ioflags, NC_WRITE))
 		return EPERM; /* attempt to write readonly file */
 
@@ -710,17 +805,17 @@ px_double_buffer(ncio *const nciop, off_t to, off_t from,
 			size_t nbytes, int rflags)
 {
 	ncio_px *const pxp = (ncio_px *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 	void *src;
 	void *dest;
-	
+
 #if INSTRUMENT
 fprintf(stderr, "\tdouble_buffr %ld %ld %ld\n",
 		 (long)to, (long)from, (long)nbytes);
 #endif
 	status = px_get(nciop, pxp, to, nbytes, RGN_WRITE,
 			&dest);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	if(pxp->slave == NULL)
@@ -731,7 +826,7 @@ fprintf(stderr, "\tdouble_buffr %ld %ld %ld\n",
 
 		pxp->slave->blksz = pxp->blksz;
 		/* pos done below */
-		pxp->slave->bf_offset = pxp->bf_offset; 
+		pxp->slave->bf_offset = pxp->bf_offset;
 		pxp->slave->bf_extent = pxp->bf_extent;
 		pxp->slave->bf_cnt = pxp->bf_cnt;
 		pxp->slave->bf_base = malloc(2 * pxp->blksz);
@@ -743,11 +838,11 @@ fprintf(stderr, "\tdouble_buffr %ld %ld %ld\n",
 		pxp->slave->bf_refcount = 0;
 		pxp->slave->slave = NULL;
 	}
-	
+
 	pxp->slave->pos = pxp->pos;
 	status = px_get(nciop, pxp->slave, from, nbytes, 0,
 			&src);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 	if(pxp->pos != pxp->slave->pos)
 	{
@@ -759,7 +854,7 @@ fprintf(stderr, "\tdouble_buffr %ld %ld %ld\n",
 
 	(void)px_rel(pxp->slave, from, 0);
 	(void)px_rel(pxp, to, RGN_MODIFIED);
-	
+
 	return status;
 }
 
@@ -782,16 +877,16 @@ ncio_px_move(ncio *const nciop, off_t to, off_t from,
 			size_t nbytes, int rflags)
 {
 	ncio_px *const pxp = (ncio_px *)nciop->pvt;
-	int status = ENOERR;
-	off_t lower;	
+	int status = NC_NOERR;
+	off_t lower;
 	off_t upper;
 	char *base;
 	size_t diff;
 	size_t extent;
 
 	if(to == from)
-		return ENOERR; /* NOOP */
-	
+		return NC_NOERR; /* NOOP */
+
 	if(fIsSet(rflags, RGN_WRITE) && !fIsSet(nciop->ioflags, NC_WRITE))
 		return EPERM; /* attempt to write readonly file */
 
@@ -800,7 +895,7 @@ ncio_px_move(ncio *const nciop, off_t to, off_t from,
 	if(to > from)
 	{
 		/* growing */
-		lower = from;	
+		lower = from;
 		upper = to;
 	}
 	else
@@ -832,7 +927,7 @@ if(to > from)
 
 			status = px_double_buffer(nciop, toh, frm,
 				 	loopextent, rflags) ;
-			if(status != ENOERR)
+			if(status != NC_NOERR)
 				return status;
 			remaining -= loopextent;
 
@@ -848,7 +943,7 @@ else
 
 			status = px_double_buffer(nciop, to, from,
 				 	loopextent, rflags) ;
-			if(status != ENOERR)
+			if(status != NC_NOERR)
 				return status;
 			remaining -= loopextent;
 
@@ -858,23 +953,23 @@ else
 			from += loopextent;
 		}
 }
-		return ENOERR;
+		return NC_NOERR;
 	}
-	
+
 #if INSTRUMENT
 fprintf(stderr, "\tncio_px_move small\n");
 #endif
 	status = px_get(nciop, pxp, lower, extent, RGN_WRITE|rflags,
 			(void **)&base);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	if(to > from)
-		(void) memmove(base + diff, base, nbytes); 
+		(void) memmove(base + diff, base, nbytes);
 	else
-		(void) memmove(base, base + diff, nbytes); 
-		
+		(void) memmove(base, base + diff, nbytes);
+
 	(void) px_rel(pxp, lower, RGN_MODIFIED);
 
 	return status;
@@ -888,14 +983,14 @@ static int
 ncio_px_sync(ncio *const nciop)
 {
 	ncio_px *const pxp = (ncio_px *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 	if(fIsSet(pxp->bf_rflags, RGN_MODIFIED))
 	{
 		assert(pxp->bf_refcount <= 0);
 		status = px_pgout(nciop, pxp->bf_offset,
 			pxp->bf_cnt,
 			pxp->bf_base, &pxp->pos);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			return status;
 		pxp->bf_rflags = 0;
 	}
@@ -933,7 +1028,7 @@ ncio_px_freepvt(void *const pvt)
 		free(pxp->slave);
 		pxp->slave = NULL;
 	}
-		
+
 	if(pxp->bf_base != NULL)
 	{
 		free(pxp->bf_base);
@@ -989,7 +1084,7 @@ ncio_px_init2(ncio *const nciop, size_t *sizehintp, int isNew)
 		pxp->bf_extent = bufsz;
 		(void) memset(pxp->bf_base, 0, pxp->bf_extent);
 	}
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -1031,7 +1126,7 @@ ncio_px_init(ncio *const nciop)
 typedef struct ncio_spx {
 	off_t pos;
 	/* buffer */
-	off_t	bf_offset; 
+	off_t	bf_offset;
 	size_t	bf_extent;
 	size_t	bf_cnt;
 	void	*bf_base;
@@ -1059,7 +1154,7 @@ static int
 ncio_spx_rel(ncio *const nciop, off_t offset, int rflags)
 {
 	ncio_spx *const pxp = (ncio_spx *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 
 	assert(pxp->bf_offset <= offset);
 	assert(pxp->bf_cnt != 0);
@@ -1108,11 +1203,11 @@ ncio_spx_get(ncio *const nciop,
 		void **const vpp)
 {
 	ncio_spx *const pxp = (ncio_spx *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 #ifdef X_ALIGN
 	size_t rem;
 #endif
-	
+
 	if(fIsSet(rflags, RGN_WRITE) && !fIsSet(nciop->ioflags, NC_WRITE))
 		return EPERM; /* attempt to write readonly file */
 
@@ -1130,9 +1225,9 @@ ncio_spx_get(ncio *const nciop,
 	}
 
 	{
-		const size_t rndup = extent % X_ALIGN;
-		if(rndup != 0)
-			extent += X_ALIGN - rndup;
+      const size_t rndup = extent % X_ALIGN;
+      if(rndup != 0)
+        extent += X_ALIGN - rndup;
 	}
 
 	assert(offset % X_ALIGN == 0);
@@ -1148,7 +1243,7 @@ ncio_spx_get(ncio *const nciop,
 			pxp->bf_extent = 0;
 		}
 		assert(pxp->bf_extent == 0);
-		pxp->bf_base = malloc(extent);
+		pxp->bf_base = malloc(extent+1);
 		if(pxp->bf_base == NULL)
 			return ENOMEM;
 		pxp->bf_extent = extent;
@@ -1158,7 +1253,7 @@ ncio_spx_get(ncio *const nciop,
 		 extent,
 		 pxp->bf_base,
 		 &pxp->bf_cnt, &pxp->pos);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	pxp->bf_offset = offset;
@@ -1171,7 +1266,7 @@ ncio_spx_get(ncio *const nciop,
 #else
 	*vpp = pxp->bf_base;
 #endif
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -1182,11 +1277,11 @@ strategy(ncio *const nciop, off_t to, off_t offset,
 			size_t extent, int rflags)
 {
 	static ncio_spx pxp[1];
-	int status = ENOERR;
+	int status = NC_NOERR;
 #ifdef X_ALIGN
 	size_t rem;
 #endif
-	
+
 	assert(extent != 0);
 	assert(extent < X_INT_MAX); /* sanity check */
 #if INSTRUMENT
@@ -1232,11 +1327,11 @@ fprintf(stderr, "strategy %ld at %ld to %ld\n",
 		 extent,
 		 pxp->bf_base,
 		 &pxp->bf_cnt, &pxp->pos);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	pxp->bf_offset = to; /* TODO: XALIGN */
-	
+
 	if(pxp->bf_cnt < extent)
 		pxp->bf_cnt = extent;
 
@@ -1265,8 +1360,8 @@ static int
 ncio_spx_move(ncio *const nciop, off_t to, off_t from,
 			size_t nbytes, int rflags)
 {
-	int status = ENOERR;
-	off_t lower = from;	
+	int status = NC_NOERR;
+	off_t lower = from;
 	off_t upper = to;
 	char *base;
 	size_t diff;
@@ -1275,12 +1370,12 @@ ncio_spx_move(ncio *const nciop, off_t to, off_t from,
 	rflags &= RGN_NOLOCK; /* filter unwanted flags */
 
 	if(to == from)
-		return ENOERR; /* NOOP */
-	
+		return NC_NOERR; /* NOOP */
+
 	if(to > from)
 	{
 		/* growing */
-		lower = from;	
+		lower = from;
 		upper = to;
 	}
 	else
@@ -1296,14 +1391,14 @@ ncio_spx_move(ncio *const nciop, off_t to, off_t from,
 	status = ncio_spx_get(nciop, lower, extent, RGN_WRITE|rflags,
 			(void **)&base);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	if(to > from)
-		(void) memmove(base + diff, base, nbytes); 
+		(void) memmove(base + diff, base, nbytes);
 	else
-		(void) memmove(base, base + diff, nbytes); 
-		
+		(void) memmove(base, base + diff, nbytes);
+
 	(void) ncio_spx_rel(nciop, lower, RGN_MODIFIED);
 
 	return status;
@@ -1317,7 +1412,7 @@ static int
 ncio_spx_sync(ncio *const nciop)
 {
 	/* NOOP */
-	return ENOERR;
+	return NC_NOERR;
 }
 
 static void
@@ -1339,12 +1434,12 @@ ncio_spx_freepvt(void *const pvt)
 
 
 /* This does the second half of the ncio_spx struct initialization for
-   POSIX systems, with NC_SHARE on. 
+   POSIX systems, with NC_SHARE on.
 
    nciop - pointer to ncio struct for this file. File has been opened.
    sizehintp - pointer to a size which will be rounded up to the
    nearest 8-byt boundary and then used as the max size "chunk" (or
-   page) to read from the file. 
+   page) to read from the file.
 */
 static int
 ncio_spx_init2(ncio *const nciop, const size_t *const sizehintp)
@@ -1365,7 +1460,7 @@ ncio_spx_init2(ncio *const nciop, const size_t *const sizehintp)
 		return ENOMEM;
 	}
 	/* else */
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -1434,7 +1529,7 @@ ncio_px_new(const char *path, int ioflags)
 	size_t sz_path = M_RNDUP(strlen(path) +1);
 	size_t sz_ncio_pvt;
 	ncio *nciop;
- 
+
 #if ALWAYS_NC_SHARE /* DEBUG */
 	fSet(ioflags, NC_SHARE);
 #endif
@@ -1447,7 +1542,7 @@ ncio_px_new(const char *path, int ioflags)
 	nciop = (ncio *) malloc(sz_ncio + sz_path + sz_ncio_pvt);
 	if(nciop == NULL)
 		return NULL;
-	
+
 	nciop->ioflags = ioflags;
 	*((int *)&nciop->fd) = -1; /* cast away const */
 
@@ -1482,15 +1577,15 @@ ncio_px_new(const char *path, int ioflags)
 #define NC_DEFAULT_CREAT_MODE 0666
 #endif
 
-/* Create a file, and the ncio struct to go with it. This funtion is
+/* Create a file, and the ncio struct to go with it. This function is
    only called from nc__create_mp.
 
    path - path of file to create.
    ioflags - flags from nc_create
    initialsz - From the netcdf man page: "The argument
    Iinitialsize sets the initial size of the file at creation time."
-   igeto - 
-   igetsz - 
+   igeto -
+   igetsz -
    sizehintp - this eventually goes into pxp->blksz and is the size of
    a page of data for buffered reads and writes.
    nciopp - pointer to a pointer that will get location of newly
@@ -1501,6 +1596,7 @@ int
 posixio_create(const char *path, int ioflags,
 	size_t initialsz,
 	off_t igeto, size_t igetsz, size_t *sizehintp,
+	void* parameters,
 	ncio **nciopp, void **const igetvpp)
 {
 	ncio *nciop;
@@ -1564,13 +1660,13 @@ posixio_create(const char *path, int ioflags,
 	else
 		status = ncio_px_init2(nciop, sizehintp, 1);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		goto unwind_open;
 
 	if(initialsz != 0)
 	{
 		status = fgrow(fd, (off_t)initialsz);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			goto unwind_open;
 	}
 
@@ -1580,12 +1676,12 @@ posixio_create(const char *path, int ioflags,
 				igeto, igetsz,
                         	RGN_WRITE,
                         	igetvpp);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			goto unwind_open;
 	}
 
 	*nciopp = nciop;
-	return ENOERR;
+	return NC_NOERR;
 
 unwind_open:
 	(void) close(fd);
@@ -1603,17 +1699,17 @@ unwind_new:
    path - path of data file.
 
    ioflags - flags passed into nc_open.
-   
+
    igeto - looks like this function can do an initial page get, and
    igeto is going to be the offset for that. But it appears to be
-   unused 
+   unused
 
    igetsz - the size in bytes of initial page get (a.k.a. extent). Not
    ever used in the library.
-   
+
    sizehintp - pointer to sizehint parameter from nc__open or
-   nc__create. This is used to set pxp->blksz. 
-   
+   nc__create. This is used to set pxp->blksz.
+
    Here's what the man page has to say:
 
    "The argument referenced by chunksize controls a space versus time
@@ -1644,6 +1740,7 @@ int
 posixio_open(const char *path,
 	int ioflags,
 	off_t igeto, size_t igetsz, size_t *sizehintp,
+        void* parameters,
 	ncio **nciopp, void **const igetvpp)
 {
 	ncio *nciop;
@@ -1659,8 +1756,10 @@ posixio_open(const char *path,
 		return ENOMEM;
 
 #ifdef O_BINARY
+	/*#if _MSC_VER*/
 	fSet(oflags, O_BINARY);
 #endif
+
 #ifdef vms
 	fd = open(path, oflags, 0, "ctx=stm");
 #else
@@ -1693,7 +1792,7 @@ posixio_open(const char *path,
 	else
 		status = ncio_px_init2(nciop, sizehintp, 0);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		goto unwind_open;
 
 	if(igetsz != 0)
@@ -1702,12 +1801,12 @@ posixio_open(const char *path,
 				igeto, igetsz,
                         	0,
                         	igetvpp);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			goto unwind_open;
 	}
 
 	*nciopp = nciop;
-	return ENOERR;
+	return NC_NOERR;
 
 unwind_open:
 	(void) close(fd); /* assert fd >= 0 */
@@ -1717,19 +1816,34 @@ unwind_new:
 	return status;
 }
 
-/* 
+/*
  * Get file size in bytes.
  */
 static int
 ncio_px_filesize(ncio *nciop, off_t *filesizep)
 {
-    struct stat sb;
 
+
+	/* There is a problem with fstat on Windows based systems
+		which manifests (so far) when Config RELEASE is built.
+		Use _filelengthi64 isntead. */
+#ifdef HAVE_FILE_LENGTH_I64
+
+	__int64 file_len = 0;
+	if( (file_len = _filelengthi64(nciop->fd)) < 0) {
+		return errno;
+	}
+
+	*filesizep = file_len;
+
+#else
+    struct stat sb;
     assert(nciop != NULL);
     if (fstat(nciop->fd, &sb) < 0)
 	return errno;
     *filesizep = sb.st_size;
-    return ENOERR;
+#endif
+	return NC_NOERR;
 }
 
 /*
@@ -1742,7 +1856,8 @@ ncio_px_filesize(ncio *nciop, off_t *filesizep)
 static int
 ncio_px_pad_length(ncio *nciop, off_t length)
 {
-	int status = ENOERR;
+
+	int status = NC_NOERR;
 
 	if(nciop == NULL)
 		return EINVAL;
@@ -1751,13 +1866,13 @@ ncio_px_pad_length(ncio *nciop, off_t length)
 	        return EPERM; /* attempt to write readonly file */
 
 	status = nciop->sync(nciop);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 	        return status;
 
  	status = fgrow2(nciop->fd, length);
- 	if(status != ENOERR)
+ 	if(status != NC_NOERR)
 	        return status;
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -1766,31 +1881,31 @@ ncio_px_pad_length(ncio *nciop, off_t length)
 
    Sync any changes, then close the open file associated with the ncio
    struct, and free its memory.
-   
+
    nciop - pointer to ncio to close.
 
    doUnlink - if true, unlink file
 */
-static int 
+static int
 ncio_px_close(ncio *nciop, int doUnlink)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 	if(nciop == NULL)
 		return EINVAL;
 	if(nciop->fd > 0) {
 	    status = nciop->sync(nciop);
 	    (void) close(nciop->fd);
-	}	
+	}
 	if(doUnlink)
 		(void) unlink(nciop->path);
 	ncio_px_free(nciop);
 	return status;
 }
 
-static int 
+static int
 ncio_spx_close(ncio *nciop, int doUnlink)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 	if(nciop == NULL)
 		return EINVAL;
 	if(nciop->fd > 0) {
