@@ -4,7 +4,10 @@
  */
 /* $Id: var.c,v 1.144 2010/05/30 00:50:35 russ Exp $ */
 
-#include "config.h"
+#if HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "nc3internal.h"
 #include <stdlib.h>
 #include <string.h>
@@ -12,10 +15,12 @@
 #include <limits.h>
 #include "ncx.h"
 #include "rnd.h"
-#include "utf8proc.h"
+#include "ncutf8.h"
 
 #ifndef OFF_T_MAX
-//#define OFF_T_MAX (~ (off_t) 0 - (~ (off_t) 0 << (CHAR_BIT * sizeof (off_t) - 1)))
+#if 0
+#define OFF_T_MAX (~ (off_t) 0 - (~ (off_t) 0 << (CHAR_BIT * sizeof (off_t) - 1)))
+#endif
 
 /* The behavior above is undefined, re: bitshifting a negative value, according
    to warnings thrown by clang/gcc.  An alternative OFF_T_MAX was written
@@ -124,9 +129,11 @@ new_NC_var(const char *uname, nc_type type,
 {
 	NC_string *strp = NULL;
 	NC_var *varp = NULL;
+	int stat;
+	char* name;
 
-	char *name = (char *)utf8proc_NFC((const unsigned char *)uname);
-	if(name == NULL)
+        stat = nc_utf8_normalize((const unsigned char *)uname,(unsigned char **)&name);
+        if(stat != NC_NOERR)
 	    return NULL;
 	strp = new_NC_string(strlen(name), name);
 	free(name);
@@ -353,6 +360,7 @@ NC_findvar(const NC_vararray *ncap, const char *uname, NC_var **varpp)
 {
 	int hash_var_id;
 	char *name;
+	int stat;
 
 	assert(ncap != NULL);
 
@@ -361,9 +369,9 @@ NC_findvar(const NC_vararray *ncap, const char *uname, NC_var **varpp)
 
 
 	/* normalized version of uname */
-	name = (char *)utf8proc_NFC((const unsigned char *)uname);
-	if(name == NULL)
-	    return NC_ENOMEM;
+        stat = nc_utf8_normalize((const unsigned char *)uname,(unsigned char **)&name);
+        if(stat != NC_NOERR)
+	    return stat;
 
 	hash_var_id = (int)NC_hashmapGetVar(ncap, name);
 	free(name);
@@ -484,6 +492,7 @@ out :
 		case NC_CHAR :
 		case NC_UBYTE :
 		case NC_SHORT :
+		case NC_USHORT :
 		        if( varp->len%4 != 0 )
 			{
 			        varp->len += 4 - varp->len%4; /* round up */
@@ -752,9 +761,9 @@ NC3_rename_var(int ncid, int varid, const char *unewname)
 
 
 	old = varp->name;
-	newname = (char *)utf8proc_NFC((const unsigned char *)unewname);
-	if(newname == NULL)
-	    return NC_ENOMEM;
+        status = nc_utf8_normalize((const unsigned char *)unewname,(unsigned char **)&newname);
+        if(status != NC_NOERR)
+	    return status;
 	if(NC_indef(ncp))
 	{
 		/* Remove old name from hashmap; add new... */
