@@ -12,21 +12,25 @@ COPYRIGHT file for copying and redistribution conditions.
 #include "config.h"
 #include <errno.h>  /* netcdf functions sometimes return system errors */
 
-
 #include "nc.h"
 #include "nc4internal.h"
 #include "nc4dispatch.h"
 
+extern int nc4_vararray_add(NC_GRP_INFO_T *grp,
+			    NC_VAR_INFO_T *var);
+
 /* must be after nc4internal.h */
 #include <H5DSpublic.h>
-
+#include <H5Fpublic.h>
 #ifdef USE_HDF4
 #include <mfhdf.h>
 #endif
-
-#ifdef USE_DISKLESS
 #include <hdf5_hl.h>
-#endif
+
+/* When we have open objects at file close, should
+   we log them or print to stdout. Default is to log
+*/
+#define LOGOPEN 1
 
 /* This is to track opened HDF5 objects to make sure they are
  * closed. */
@@ -38,18 +42,81 @@ extern int num_spaces;
 #define MIN_DEFLATE_LEVEL 0
 #define MAX_DEFLATE_LEVEL 9
 
-/* These are the special attributes added by the HDF5 dimension scale
- * API. They will be ignored by netCDF-4. */
-#define REFERENCE_LIST "REFERENCE_LIST"
-#define CLASS "CLASS"
-#define DIMENSION_LIST "DIMENSION_LIST"
-#define NAME "NAME"
+/*Forward*/
+static int read_hdf5_att(NC_GRP_INFO_T *grp, hid_t attid, NC_ATT_INFO_T *att);
+static void hdf5free(void* memory);
+
+/* Custom iteration callback data */
+typedef struct {
+  NC_GRP_INFO_T *grp;
+  NC_VAR_INFO_T *var;
+} att_iter_info;
+
+static herr_t
+att_read_var_callbk(hid_t loc_id, const char *att_name, const H5A_info_t *ainfo, void *att_data)
+{
+
+  hid_t attid = 0;
+  int retval = NC_NOERR;
+  NC_ATT_INFO_T *att;
+  att_iter_info *att_info = (att_iter_info *)att_data;
+  const char** reserved;
+
+
+  /* Should we ignore this attribute? */
+  for(reserved=NC_RESERVED_VARATT_LIST;*reserved;reserved++) {
+    if (strcmp(att_name, *reserved)==0) break;
+  }
+
+  if(*reserved == NULL) {
+      /* Open the att by name. */
+      if ((attid = H5Aopen(loc_id, att_name, H5P_DEFAULT)) < 0)
+	BAIL(NC_EATTMETA);
+      LOG((4, "%s::  att_name %s", __func__, att_name));
+      /* Add to the end of the list of atts for this var. */
+      if ((retval = nc4_att_list_add(&att_info->var->att, &att)))
+	BAIL(retval);
+      /* Fill in the information we know. */
+      att->attnum = att_info->var->natts++;
+      if (!(att->name = strdup(att_name)))
+	BAIL(NC_ENOMEM);
+
+      /* Read the rest of the info about the att,
+       * including its values. */
+      if ((retval = read_hdf5_att(att_info->grp, attid, att)))
+	{
+	  if (NC_EBADTYPID == retval)
+            {
+	      if ((retval = nc4_att_list_del(&att_info->var->att, att)))
+		BAIL(retval);
+            }
+	  else
+	    BAIL(retval);
+	}
+
+      att->created = NC_TRUE;
+
+      if (attid > 0 && H5Aclose(attid) < 0)
+	BAIL2(NC_EHDFERR);
+
+    } /* endif not HDF5 att */
+  
+  return NC_NOERR;
+
+ exit:
+  if (attid > 0 && H5Aclose(attid) < 0)
+    BAIL2(NC_EHDFERR);
+
+  return retval;
+}
 
 /* Define the illegal mode flags */
 static const int ILLEGAL_OPEN_FLAGS = (NC_MMAP|NC_64BIT_OFFSET);
 
 static const int ILLEGAL_CREATE_FLAGS = (NC_NOWRITE|NC_MMAP|NC_INMEMORY|NC_64BIT_OFFSET|NC_CDF5);
 
+extern void reportopenobjects(int log, hid_t);
+
 /*! Struct to track information about objects in a group, for nc4_rec_read_metadata()
 
  \internal
@@ -80,16 +147,48 @@ static int NC4_enddef(int ncid);
 static int nc4_rec_read_metadata(NC_GRP_INFO_T *grp);
 static int close_netcdf4_file(NC_HDF5_FILE_INFO_T *h5, int abort);
 
+/* Define the names of attributes to ignore
+ * added by the HDF5 dimension scale; these
+ * attached to variables.
+ * They cannot be modified thru the netcdf-4 API.
+ */
+const char* NC_RESERVED_VARATT_LIST[] = {
+NC_ATT_REFERENCE_LIST,
+NC_ATT_CLASS,
+NC_ATT_DIMENSION_LIST,
+NC_ATT_NAME,
+NC_ATT_COORDINATES,
+NC_DIMID_ATT_NAME,
+NULL
+};
+
+/* Define the names of attributes to ignore
+ * because they are "hidden" global attributes.
+ * They can be read, but not modified thru the netcdf-4 API.
+ */
+const char* NC_RESERVED_ATT_LIST[] = {
+NC_ATT_FORMAT,
+NC3_STRICT_ATT_NAME,
+NCPROPS,
+ISNETCDF4ATT,
+SUPERBLOCKATT,
+NULL
+};
+
+/* Define the subset of the reserved list that is readable by name only */
+const char* NC_RESERVED_SPECIAL_LIST[] = {
+ISNETCDF4ATT,
+SUPERBLOCKATT,
+NCPROPS,
+NULL
+};
+
 /* These are the default chunk cache sizes for HDF5 files created or
  * opened with netCDF-4. */
 size_t nc4_chunk_cache_size = CHUNK_CACHE_SIZE;
 size_t nc4_chunk_cache_nelems = CHUNK_CACHE_NELEMS;
 float nc4_chunk_cache_preemption = CHUNK_CACHE_PREEMPTION;
 
-/* To turn off HDF5 error messages, I have to catch an early
-   invocation of a netcdf function. */
-static int virgin = 1;
-
 /* For performance, fill this array only the first time, and keep it
  * in global memory for each further use. */
 #define NUM_TYPES 12
@@ -200,9 +299,7 @@ nc_check_for_hdf(const char *path, int flags, void* parameters, int *hdf_file)
    MPI_Info info = MPI_INFO_NULL;
 #endif
    int inmemory = ((flags & NC_INMEMORY) == NC_INMEMORY);
-#ifdef USE_DISKLESS
    NC_MEM_INFO* meminfo = (NC_MEM_INFO*)parameters;
-#endif
 
 #ifdef USE_PARALLEL4
    if(use_parallel) {
@@ -313,12 +410,7 @@ nc4_create_file(const char *path, int cmode, MPI_Comm comm, MPI_Info info,
    nc4_info = NC4_DATA(nc);
    assert(nc4_info && nc4_info->root_grp);
 
-#if 0 /*def USE_PNETCDF*/
-    if (cmode & NC_PNETCDF)
-	return NC_NOERR;
-#endif
-
-   /* Need this access plist to control how HDF5 handles open onjects
+   /* Need this access plist to control how HDF5 handles open objects
     * on file close. (Setting H5F_CLOSE_SEMI will cause H5Fclose to
     * fail if there are any open objects in the file. */
    if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0)
@@ -390,8 +482,10 @@ nc4_create_file(const char *path, int cmode, MPI_Comm comm, MPI_Info info,
 	__func__, nc4_chunk_cache_size, nc4_chunk_cache_nelems, nc4_chunk_cache_preemption));
 #endif /* USE_PARALLEL4 */
 
-   if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0)
+#ifdef HDF5_HAS_LIBVER_BOUNDS
+   if (H5Pset_libver_bounds(fapl_id, H5F_LIBVER_EARLIEST, H5F_LIBVER_LATEST) < 0)
       BAIL(NC_EHDFERR);
+#endif
 
    /* Create the property list. */
    if ((fcpl_id = H5Pcreate(H5P_FILE_CREATE)) < 0)
@@ -415,6 +509,11 @@ nc4_create_file(const char *path, int cmode, MPI_Comm comm, MPI_Info info,
       BAIL(NC_EHDFERR);
 
    /* Create the file. */
+#ifdef HDF5_HAS_COLL_METADATA_OPS
+   H5Pset_all_coll_metadata_ops(fapl_id, 1 );
+   H5Pset_coll_metadata_write(fapl_id, 1);
+#endif
+
    if ((nc4_info->hdfid = H5Fcreate(path, flags, fcpl_id, fapl_id)) < 0)
         /*Change the return error from NC_EFILEMETADATA to
           System error EACCES because that is the more likely problem */
@@ -426,8 +525,7 @@ nc4_create_file(const char *path, int cmode, MPI_Comm comm, MPI_Info info,
       BAIL(NC_EFILEMETA);
 
    /* Release the property lists. */
-   if (H5Pclose(fapl_id) < 0 ||
-       H5Pclose(fcpl_id) < 0)
+   if (H5Pclose(fapl_id) < 0 || H5Pclose(fcpl_id) < 0)
       BAIL(NC_EHDFERR);
 #ifdef EXTRA_TESTS
    num_plists--;
@@ -437,6 +535,9 @@ nc4_create_file(const char *path, int cmode, MPI_Comm comm, MPI_Info info,
    /* Define mode gets turned on automatically on create. */
    nc4_info->flags |= NC_INDEF;
 
+   NC4_get_fileinfo(nc4_info,&globalpropinfo);
+   NC4_put_propattr(nc4_info);
+
    return NC_NOERR;
 
 exit: /*failure exit*/
@@ -491,25 +592,20 @@ NC4_create(const char* path, int cmode, size_t initialsz, int basepe,
 #endif /* USE_PARALLEL4 */
 
    /* If this is our first file, turn off HDF5 error messages. */
-   if (virgin)
-   {
-      if (H5Eset_auto(NULL, NULL) < 0)
-	 LOG((0, "Couldn't turn off HDF5 error messages!"));
-      LOG((1, "HDF5 error messages have been turned off."));
-      virgin = 0;
-   }
+   if (!nc4_hdf5_initialized)
+	nc4_hdf5_initialize();
 
    /* Check the cmode for validity. */
    if((cmode & ILLEGAL_CREATE_FLAGS) != 0)
-      return NC_EINVAL;
+      {res = NC_EINVAL; goto done;}
 
    /* Cannot have both */
    if((cmode & (NC_MPIIO|NC_MPIPOSIX)) == (NC_MPIIO|NC_MPIPOSIX))
-      return NC_EINVAL;
+      {res = NC_EINVAL; goto done;}
 
    /* Currently no parallel diskless io */
    if((cmode & (NC_MPIIO | NC_MPIPOSIX)) && (cmode & NC_DISKLESS))
-      return NC_EINVAL;
+      {res = NC_EINVAL; goto done;}
 
 #ifndef USE_PARALLEL_POSIX
 /* If the HDF5 library has been compiled without the MPI-POSIX VFD, alias
@@ -535,20 +631,10 @@ NC4_create(const char* path, int cmode, size_t initialsz, int basepe,
    LOG((2, "cmode after applying default format: 0x%x", cmode));
 
    nc_file->int_ncid = nc_file->ext_ncid;
+
    res = nc4_create_file(path, cmode, comm, info, nc_file);
 
-#if 0 /*def USE_PNETCDF*/
-   if (cmode & NC_PNETCDF)
-   {
-      NC_HDF5_FILE_INFO_T* nc4_info;
-      nc4_info = NC4_DATA(nc_file);
-      assert(nc4_info);
-
-      nc4_info->pnetcdf_file++;
-      res = ncmpi_create(comm, path, cmode, info, &(nc_file->int_ncid));
-   }
-#endif /* USE_PNETCDF */
-
+done:
    return res;
 }
 
@@ -567,7 +653,6 @@ read_scale(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
    htri_t attr_exists = -1;             /* Flag indicating hidden attribute exists */
    hid_t attid = -1;                    /* ID of hidden attribute (to store dim ID) */
    int dimscale_created = 0;            /* Remember if a dimension was created (for error recovery) */
-   int initial_grp_ndims = grp->ndims;  /* Retain for error recovery */
    short initial_next_dimid = grp->nc4_info->next_dimid;/* Retain for error recovery */
    int retval;
 
@@ -582,8 +667,7 @@ read_scale(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
       BAIL(NC_EHDFERR);
    if (attr_exists)
    {
-      if ((attid = H5Aopen_by_name(datasetid, ".", NC_DIMID_ATT_NAME,
-				   H5P_DEFAULT, H5P_DEFAULT)) < 0)
+      if ((attid = H5Aopen_name(datasetid, NC_DIMID_ATT_NAME)) < 0)
          BAIL(NC_EHDFERR);
 
       if (H5Aread(attid, H5T_NATIVE_INT, &new_dim->dimid) < 0)
@@ -599,9 +683,6 @@ read_scale(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
       new_dim->dimid = grp->nc4_info->next_dimid++;
    }
 
-   /* Increment number of dimensions. */
-   grp->ndims++;
-
    if (!(new_dim->name = strdup(obj_name)))
       BAIL(NC_ENOMEM);
    if (SIZEOF_SIZE_T < 8 && scale_size > NC_MAX_UINT)
@@ -615,6 +696,7 @@ read_scale(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
    new_dim->hdf5_objid.fileno[1] = statbuf->fileno[1];
    new_dim->hdf5_objid.objno[0] = statbuf->objno[0];
    new_dim->hdf5_objid.objno[1] = statbuf->objno[1];
+   new_dim->hash = hash_fast(obj_name, strlen(obj_name));
 
    /* If the dimscale has an unlimited dimension, then this dimension
     * is unlimited. */
@@ -660,7 +742,6 @@ exit:
            BAIL2(retval);
 
       /* Reset the group's information */
-      grp->ndims = initial_grp_ndims;
       grp->nc4_info->next_dimid = initial_next_dimid;
    }
 
@@ -670,11 +751,12 @@ exit:
 /* This function reads the hacked in coordinates attribute I use for
  * multi-dimensional coordinates. */
 static int
-read_coord_dimids(NC_VAR_INFO_T *var)
+read_coord_dimids(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var)
 {
    hid_t coord_att_typeid = -1, coord_attid = -1, spaceid = -1;
    hssize_t npoints;
    int ret = 0;
+   int d;
 
    /* There is a hidden attribute telling us the ids of the
     * dimensions that apply to this multi-dimensional coordinate
@@ -696,6 +778,12 @@ read_coord_dimids(NC_VAR_INFO_T *var)
    if (!ret && H5Aread(coord_attid, coord_att_typeid, var->dimids) < 0) ret++;
    LOG((4, "dimscale %s is multidimensional and has coords", var->name));
 
+   /* Update var->dim field based on the var->dimids */
+   for (d = 0; d < var->ndims; d++) {
+     /* Ok if does not find a dim at this time, but if found set it */
+     nc4_find_dim(grp, var->dimids[d], &var->dim[d], NULL);
+   }
+
    /* Set my HDF5 IDs free! */
    if (spaceid >= 0 && H5Sclose(spaceid) < 0) ret++;
 #ifdef EXTRA_TESTS
@@ -1309,27 +1397,11 @@ read_type(NC_GRP_INFO_T *grp, hid_t hdf_typeid, char *type_name)
                      break;
                }
 
-#ifndef JNA
-
-               /* Free the member name (which HDF5 allocated for us). */
-			   /* On Windows using the microsoft runtime, it is an error
-				  for one library to free memory allocated by a different library.
-                  IF it is available, we should use H5free_memory*/
-
-#ifdef HDF5_HAS_H5FREE
-               if(member_name != NULL) H5free_memory(member_name);
-#else
-#ifndef _MSC_VER
-               if(member_name != NULL) free(member_name);
-#endif
-#endif
-#endif
+	       hdf5free(member_name);
 	       member_name = NULL;
             }
-#ifndef JNA
-	    if(member_name != NULL)
-		free(member_name);
-#endif
+	    hdf5free(member_name);
+	    member_name = NULL;	    	
 	    if(retval) /* error exit from loop */
 		return retval;
          }
@@ -1453,17 +1525,11 @@ read_type(NC_GRP_INFO_T *grp, hid_t hdf_typeid, char *type_name)
 		  break;
                }
 
-#ifndef JNA
-               /* Free the member name (which HDF5 allocated for us). */
-               if(member_name != NULL) free(member_name);
-#endif
+	       hdf5free(member_name);
 	       member_name = NULL;
             }
-
-#ifndef JNA
-	    if(member_name != NULL)
-		free(member_name);
-#endif
+	    hdf5free(member_name);
+	    member_name = NULL;
         if(value) free(value);
         if(retval) /* error exit from loop */
           return retval;
@@ -1489,9 +1555,10 @@ read_var(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
    hid_t access_pid = 0;
    int incr_id_rc = 0;          /* Whether the dataset ID's ref count has been incremented */
    int natts, a, d;
+   const char** reserved;
 
    NC_ATT_INFO_T *att;
-   hid_t attid = 0;
+   att_iter_info att_info;         /* Custom iteration information */
    char att_name[NC_MAX_HDF5_NAME + 1];
 
 #define CD_NELEMS_ZLIB 1
@@ -1512,7 +1579,7 @@ read_var(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
    LOG((4, "%s: obj_name %s", __func__, obj_name));
 
    /* Add a variable to the end of the group's var list. */
-   if ((retval = nc4_var_list_add(&grp->var, &var)))
+   if ((retval = nc4_var_add(&var)))
       BAIL(retval);
 
    /* Fill in what we already know. */
@@ -1573,6 +1640,7 @@ read_var(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
       strcpy(var->name, obj_name);
    }
 
+   var->hash = hash_fast(var->name, strlen(var->name));
    /* Find out what filters are applied to this HDF5 dataset,
     * fletcher32, deflate, and/or shuffle. All other filters are
     * ignored. */
@@ -1688,7 +1756,7 @@ read_var(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
       var->dimscale = NC_TRUE;
       if (var->ndims > 1)
       {
-	 if ((retval = read_coord_dimids(var)))
+	 if ((retval = read_coord_dimids(grp, var)))
 	    BAIL(retval);
       }
       else
@@ -1738,57 +1806,14 @@ read_var(NC_GRP_INFO_T *grp, hid_t datasetid, const char *obj_name,
 
    /* Now read all the attributes of this variable, ignoring the
       ones that hold HDF5 dimension scale information. */
-   if ((natts = H5Aget_num_attrs(var->hdf_datasetid)) < 0)
-      BAIL(NC_EATTMETA);
-   for (a = 0; a < natts; a++)
-   {
-      /* Close the attribute and try to move on with our
-       * lives. Like bits through the network port, so
-       * flows the Days of Our Lives! */
-      if (attid && H5Aclose(attid) < 0)
-         BAIL(NC_EHDFERR);
 
-      /* Open the att and get its name. */
-      if ((attid = H5Aopen_idx(var->hdf_datasetid, (unsigned int)a)) < 0)
-         BAIL(NC_EATTMETA);
-      if (H5Aget_name(attid, NC_MAX_HDF5_NAME, att_name) < 0)
-         BAIL(NC_EATTMETA);
-      LOG((4, "%s:: a %d att_name %s", __func__, a, att_name));
+   att_info.var = var;
+   att_info.grp = grp;
 
-      /* Should we ignore this attribute? */
-      if (strcmp(att_name, REFERENCE_LIST) &&
-	  strcmp(att_name, CLASS) &&
-	  strcmp(att_name, DIMENSION_LIST) &&
-	  strcmp(att_name, NAME) &&
-	  strcmp(att_name, COORDINATES) &&
-	  strcmp(att_name, NC_DIMID_ATT_NAME))
-      {
-	 /* Add to the end of the list of atts for this var. */
-	 if ((retval = nc4_att_list_add(&var->att, &att)))
-	    BAIL(retval);
+   if ((H5Aiterate2(var->hdf_datasetid, H5_INDEX_CRT_ORDER, H5_ITER_INC, NULL, att_read_var_callbk, &att_info)) < 0)
+     BAIL(NC_EATTMETA);
 
-	 /* Fill in the information we know. */
-	 att->attnum = var->natts++;
-	 if (!(att->name = strdup(att_name)))
-	    BAIL(NC_ENOMEM);
-
-	 /* Read the rest of the info about the att,
-	  * including its values. */
-	 if ((retval = read_hdf5_att(grp, attid, att)))
-         {
-            if (NC_EBADTYPID == retval)
-            {
-                if ((retval = nc4_att_list_del(&var->att, att)))
-                    BAIL(retval);
-                continue;
-            }
-            else
-                BAIL(retval);
-         }
-
-	 att->created = NC_TRUE;
-      } /* endif not HDF5 att */
-   } /* next attribute */
+   nc4_vararray_add(grp, var);
 
    /* Is this a deflated variable with a chunksize greater than the
     * current cache size? */
@@ -1800,7 +1825,7 @@ exit:
    {
        if (incr_id_rc && H5Idec_ref(datasetid) < 0)
           BAIL2(NC_EHDFERR);
-       if (var && nc4_var_list_del(&grp->var, var))
+       if (var && nc4_var_del(var))
           BAIL2(NC_EHDFERR);
    }
    if (access_pid && H5Pclose(access_pid) < 0)
@@ -1813,8 +1838,6 @@ exit:
 #ifdef EXTRA_TESTS
    num_plists--;
 #endif
-   if (attid > 0 && H5Aclose(attid) < 0)
-      BAIL2(NC_EHDFERR);
    return retval;
 }
 
@@ -1823,36 +1846,43 @@ exit:
 static int
 read_grp_atts(NC_GRP_INFO_T *grp)
 {
-   hid_t attid = 0;
+   hid_t attid = -1;
    hsize_t num_obj, i;
    NC_ATT_INFO_T *att;
    NC_TYPE_INFO_T *type;
    char obj_name[NC_MAX_HDF5_NAME + 1];
    int max_len;
    int retval = NC_NOERR;
+   int hidden = 0;
 
    num_obj = H5Aget_num_attrs(grp->hdf_grpid);
    for (i = 0; i < num_obj; i++)
    {
-      /* Close an attribute from previous loop iteration */
-      /* (Should be from 'continue' statement, below) */
-      if (attid && H5Aclose(attid) < 0)
-         BAIL(NC_EHDFERR);
-
       if ((attid = H5Aopen_idx(grp->hdf_grpid, (unsigned int)i)) < 0)
          BAIL(NC_EATTMETA);
       if (H5Aget_name(attid, NC_MAX_NAME + 1, obj_name) < 0)
          BAIL(NC_EATTMETA);
       LOG((3, "reading attribute of _netCDF group, named %s", obj_name));
 
+      /* See if this a hidden, global attribute */
+      if(grp->nc4_info->root_grp == grp) {
+	const char** reserved = NC_RESERVED_ATT_LIST;
+	hidden = 0;
+	for(;*reserved;reserved++) {
+	    if(strcmp(*reserved,obj_name)==0) {
+		hidden = 1;
+		break;
+	    }
+	}
+      }
+
       /* This may be an attribute telling us that strict netcdf-3
        * rules are in effect. If so, we will make note of the fact,
        * but not add this attribute to the metadata. It's not a user
        * attribute, but an internal netcdf-4 one. */
-      if (!strcmp(obj_name, NC3_STRICT_ATT_NAME))
-         grp->nc4_info->cmode |= NC_CLASSIC_MODEL;
-      else
-      {
+      if(strcmp(obj_name, NC3_STRICT_ATT_NAME)==0)
+             grp->nc4_info->cmode |= NC_CLASSIC_MODEL;
+      else if(!hidden) {
          /* Add an att struct at the end of the list, and then go to it. */
          if ((retval = nc4_att_list_add(&grp->att, &att)))
             BAIL(retval);
@@ -1864,26 +1894,28 @@ read_grp_atts(NC_GRP_INFO_T *grp)
          strncpy(att->name, obj_name, max_len);
          att->name[max_len] = 0;
          att->attnum = grp->natts++;
-         if ((retval = read_hdf5_att(grp, attid, att)))
-         {
-            if (NC_EBADTYPID == retval)
-            {
-               if ((retval = nc4_att_list_del(&grp->att, att)))
+         retval = read_hdf5_att(grp, attid, att);
+         if(retval == NC_EBADTYPID) {
+               if((retval = nc4_att_list_del(&grp->att, att)))
                   BAIL(retval);
-               continue;
-            }
-            else
+	 } else if(retval) {
                BAIL(retval);
-         }
-         att->created = NC_TRUE;
-         if ((retval = nc4_find_type(grp->nc4_info, att->nc_typeid, &type)))
-            BAIL(retval);
+         } else {
+             att->created = NC_TRUE;
+	     if ((retval = nc4_find_type(grp->nc4_info, att->nc_typeid, &type)))
+            	BAIL(retval);
+	 }
       }
+      /* Unconditionally close the open attribute */
+      H5Aclose(attid);
+      attid = -1;
    }
 
-  exit:
-   if (attid > 0 && H5Aclose(attid) < 0)
-      BAIL2(NC_EHDFERR);
+exit:
+   if (attid > 0) {
+	if(H5Aclose(attid) < 0)
+            BAIL2(NC_EHDFERR);
+   }
    return retval;
 }
 
@@ -2073,7 +2105,7 @@ nc4_rec_read_metadata(NC_GRP_INFO_T *grp)
     hid_t pid = 0;
     unsigned crt_order_flags = 0;
     H5_index_t iter_index;
-    int retval = NC_NOERR; /* everything worked! */
+    int i, retval = NC_NOERR; /* everything worked! */
 
     assert(grp && grp->name);
     LOG((3, "%s: grp->name %s", __func__, grp->name));
@@ -2161,6 +2193,10 @@ nc4_rec_read_metadata(NC_GRP_INFO_T *grp)
     if ((retval = read_grp_atts(grp)))
 	BAIL(retval);
 
+   /* when exiting define mode, mark all variable written */
+   for (i=0; i<grp->vars.nelems; i++)
+      grp->vars.value[i]->written_to = NC_TRUE;
+
 exit:
     /* Clean up local information on error, if anything remains */
     if (retval)
@@ -2192,9 +2228,7 @@ nc4_open_file(const char *path, int mode, void* parameters, NC *nc)
    int retval;
    NC_HDF5_FILE_INFO_T* nc4_info = NULL;
    int inmemory = ((mode & NC_INMEMORY) == NC_INMEMORY);
-#ifdef USE_DISKLESS
    NC_MEM_INFO* meminfo = (NC_MEM_INFO*)parameters;
-#endif
 #ifdef USE_PARALLEL4
    NC_MPI_INFO* mpiinfo = (NC_MPI_INFO*)parameters;
    int comm_duped = 0;          /* Whether the MPI Communicator was duplicated */
@@ -2215,6 +2249,7 @@ nc4_open_file(const char *path, int mode, void* parameters, NC *nc)
     * fail if there are any open objects in the file. */
    if ((fapl_id = H5Pcreate(H5P_FILE_ACCESS)) < 0)
       BAIL(NC_EHDFERR);
+
 #ifdef EXTRA_TESTS
    num_plists++;
 #endif
@@ -2281,6 +2316,9 @@ nc4_open_file(const char *path, int mode, void* parameters, NC *nc)
    /* The NetCDF-3.x prototype contains an mode option NC_SHARE for
       multiple processes accessing the dataset concurrently.  As there
       is no HDF5 equivalent, NC_SHARE is treated as NC_NOWRITE. */
+#ifdef HDF5_HAS_COLL_METADATA_OPS
+   H5Pset_all_coll_metadata_ops(fapl_id, 1 );
+#endif
    if(inmemory) {
        if((nc4_info->hdfid = H5LTopen_file_image(meminfo->memory,meminfo->size,
 			H5LT_FILE_IMAGE_DONT_COPY|H5LT_FILE_IMAGE_DONT_RELEASE
@@ -2319,6 +2357,8 @@ nc4_open_file(const char *path, int mode, void* parameters, NC *nc)
    num_plists--;
 #endif
 
+   NC4_get_fileinfo(nc4_info,NULL);
+
    return NC_NOERR;
 
 exit:
@@ -2558,14 +2598,16 @@ nc4_open_hdf4_file(const char *path, int mode, NC *nc)
       size_t var_type_size;
       int a;
 
-      /* Add a variable to the end of the group's var list. */
-      if ((retval = nc4_var_list_add(&grp->var, &var)))
+      /* Add a variable. */
+      if ((retval = nc4_var_add(&var)))
 	return retval;
 
       var->varid = grp->nvars++;
       var->created = NC_TRUE;
       var->written_to = NC_TRUE;
 
+      nc4_vararray_add(grp, var);
+
       /* Open this dataset in HDF4 file. */
       if ((var->sdsid = SDselect(h5->sdid, v)) == FAIL)
 	return NC_EVARMETA;
@@ -2578,6 +2620,8 @@ nc4_open_hdf4_file(const char *path, int mode, NC *nc)
       if (SDgetinfo(var->sdsid, var->name, &rank, NULL, &data_type, &num_atts))
 	return NC_EVARMETA;
 
+      var->hash = hash_fast(var->name, strlen(var->name));
+
       if(!(dimsize = (int32*)malloc(sizeof(int32)*rank)))
 	return NC_ENOMEM;
 
@@ -2671,7 +2715,6 @@ nc4_open_hdf4_file(const char *path, int mode, NC *nc)
 		 dim_name, var->name));
 	    if ((retval = nc4_dim_list_add(&grp->dim, &dim)))
 	       return retval;
-	    grp->ndims++;
 	    dim->dimid = grp->nc4_info->next_dimid++;
 	    if (strlen(dim_name) > NC_MAX_HDF4_NAME)
 	       return NC_EMAXNAME;
@@ -2681,10 +2724,12 @@ nc4_open_hdf4_file(const char *path, int mode, NC *nc)
 	       dim->len = dim_len;
 	    else
 	       dim->len = *dimsize;
+	    dim->hash = hash_fast(dim_name, strlen(dim_name));
 	 }
 
 	 /* Tell the variable the id of this dimension. */
 	 var->dimids[d] = dim->dimid;
+	 var->dim[d] = dim;
       }
 
       /* Read the atts. */
@@ -2789,23 +2834,17 @@ NC4_open(const char *path, int mode, int basepe, size_t *chunksizehintp,
 	parameters = &mpidfalt;
 #endif /* USE_PARALLEL4 */
 
-   /* If this is our first file, turn off HDF5 error messages. */
-   if (virgin)
-   {
-      if (H5Eset_auto(NULL, NULL) < 0)
-	 LOG((0, "Couldn't turn off HDF5 error messages!"));
-      LOG((1, "HDF5 error messages turned off!"));
-      virgin = 0;
-   }
-
+   /* If this is our first file, initialize HDF5. */
+   if (!nc4_hdf5_initialized)
+	nc4_hdf5_initialize();
 
    /* Check the mode for validity */
    if((mode & ILLEGAL_OPEN_FLAGS) != 0)
-      return NC_EINVAL;
+      {res = NC_EINVAL; goto done;}
 
    /* Cannot have both */
    if((mode & (NC_MPIIO|NC_MPIPOSIX)) == (NC_MPIIO|NC_MPIPOSIX))
-      return NC_EINVAL;
+      {res = NC_EINVAL; goto done;}
 
 #ifndef USE_PARALLEL_POSIX
 /* If the HDF5 library has been compiled without the MPI-POSIX VFD, alias
@@ -2820,7 +2859,7 @@ NC4_open(const char *path, int mode, int basepe, size_t *chunksizehintp,
 
    /* Figure out if this is a hdf4 or hdf5 file. */
    if ((res = nc_check_for_hdf(path, use_parallel, parameters, &hdf_file)))
-	return res;
+	goto done;
 
    /* Depending on the type of file, open it. */
    nc_file->int_ncid = nc_file->ext_ncid;
@@ -2828,13 +2867,13 @@ NC4_open(const char *path, int mode, int basepe, size_t *chunksizehintp,
        res = nc4_open_file(path, mode, parameters, nc_file);
 #ifdef USE_HDF4
    else if (hdf_file == NC_HDF4_FILE && inmemory)
-	return NC_EDISKLESS;
+	{res = NC_EDISKLESS; goto done;}
    else if (hdf_file == NC_HDF4_FILE)
        res = nc4_open_hdf4_file(path, mode, nc_file);
 #endif /* USE_HDF4 */
    else
-         assert(0); /* should never happen */
-
+       assert(0); /* should never happen */
+done:
    return res;
 }
 
@@ -2870,12 +2909,6 @@ NC4_set_fill(int ncid, int fillmode, int *old_modep)
 
    nc4_info->fill_mode = fillmode;
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (nc4_info->pnetcdf_file)
-     return ncmpi_set_fill(nc->int_ncid, fillmode, old_modep);
-#endif /* USE_PNETCDF */
-
 
    return NC_NOERR;
 }
@@ -2894,12 +2927,6 @@ NC4_redef(int ncid)
       return NC_EBADID;
    assert(nc4_info);
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (nc4_info->pnetcdf_file)
-      return ncmpi_redef(nc->int_ncid);
-#endif /* USE_PNETCDF */
-
    /* If we're already in define mode, return an error. */
    if (nc4_info->flags & NC_INDEF)
       return NC_EINDEFINE;
@@ -2936,6 +2963,8 @@ static int NC4_enddef(int ncid)
 {
   NC *nc;
    NC_HDF5_FILE_INFO_T* nc4_info;
+   NC_GRP_INFO_T *grp;
+   int i;
 
    LOG((1, "%s: ncid 0x%x", __func__, ncid));
 
@@ -2943,19 +2972,13 @@ static int NC4_enddef(int ncid)
       return NC_EBADID;
    assert(nc4_info);
 
-#if 0 /*def USE_PNETCDF*/
-   if (nc4_info->pnetcdf_file)
-   {
-      int res;
-      res = ncmpi_enddef(nc->int_ncid);
-      if (!res)
-      {
-	 if (nc4_info->pnetcdf_access_mode == NC_INDEPENDENT)
-	    res = ncmpi_begin_indep_data(nc->int_ncid);
-      }
-      return res;
-   }
-#endif /* USE_PNETCDF */
+   /* Find info for this file and group */
+   if (!(grp = nc4_rec_find_grp(nc4_info->root_grp, (ncid & GRP_ID_MASK))))
+      return NC_EBADGRPID;
+
+   /* when exiting define mode, mark all variable written */
+   for (i=0; i<grp->vars.nelems; i++)
+      grp->vars.value[i]->written_to = NC_TRUE;
 
    return nc4_enddef_netcdf4_file(nc4_info);
 }
@@ -3024,12 +3047,6 @@ NC4_sync(int ncid)
       return NC_EBADID;
    assert(nc4_info);
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (nc4_info->pnetcdf_file)
-      return ncmpi_sync(nc->int_ncid);
-#endif /* USE_PNETCDF */
-
    /* If we're in define mode, we can't sync. */
    if (nc4_info && nc4_info->flags & NC_INDEF)
    {
@@ -3082,11 +3099,15 @@ close_netcdf4_file(NC_HDF5_FILE_INFO_T *h5, int abort)
       /* Free the MPI Comm & Info objects, if we opened the file in parallel */
       if(h5->parallel)
       {
-          MPI_Comm_free(&h5->comm);
+          if(MPI_COMM_NULL != h5->comm)
+              MPI_Comm_free(&h5->comm);
           if(MPI_INFO_NULL != h5->info)
               MPI_Info_free(&h5->info);
       }
 #endif
+
+      if(h5->fileinfo) free(h5->fileinfo);
+
       if (H5Fclose(h5->hdfid) < 0)
       {
 	int nobjs;
@@ -3097,23 +3118,29 @@ close_netcdf4_file(NC_HDF5_FILE_INFO_T *h5, int abort)
           BAIL_QUIET(NC_EHDFERR);
 	} else if(nobjs > 0) {
 #ifdef LOGGING
+	 char msg[1024];
+	 int logit = 1;
 	 /* If the close doesn't work, probably there are still some HDF5
 	  * objects open, which means there's a bug in the library. So
 	  * print out some info on to help the poor programmer figure it
 	  * out. */
-         LOG((0, "There are %d HDF5 objects open!", nobjs));
+	snprintf(msg,sizeof(msg),"There are %d HDF5 objects open!", nobjs);
+#ifdef LOGOPEN
+         LOG((0, msg));
+#else
+         fprintf(stdout,msg);
+	 logit = 0;
+#endif
+	 reportopenobjects(logit,h5->hdfid);
 #endif
-         BAIL_QUIET(NC_EHDFERR);
-	}
       }
+    }
    }
-
 exit:
    /* Free the nc4_info struct; above code should have reclaimed
       everything else */
    if(h5 != NULL)
        free(h5);
-
    return retval;
 }
 
@@ -3140,12 +3167,6 @@ NC4_abort(int ncid)
 
    assert(nc4_info);
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (nc4_info->pnetcdf_file)
-      return ncmpi_abort(nc->int_ncid);
-#endif /* USE_PNETCDF */
-
    /* If we're in define mode, but not redefing the file, delete it. */
    if (nc4_info->flags & NC_INDEF && !nc4_info->redef)
    {
@@ -3187,12 +3208,6 @@ NC4_close(int ncid)
    if (grp->parent)
       return NC_EBADGRPID;
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (h5->pnetcdf_file)
-      return ncmpi_close(nc->int_ncid);
-#endif /* USE_PNETCDF */
-
    /* Call the nc4 close. */
    if ((retval = close_netcdf4_file(grp->nc4_info, 0)))
       return retval;
@@ -3210,7 +3225,6 @@ NC4_inq(int ncid, int *ndimsp, int *nvarsp, int *nattsp, int *unlimdimidp)
    NC_GRP_INFO_T *grp;
    NC_DIM_INFO_T *dim;
    NC_ATT_INFO_T *att;
-   NC_VAR_INFO_T *var;
    int retval;
 
    LOG((2, "%s: ncid 0x%x", __func__, ncid));
@@ -3221,12 +3235,6 @@ NC4_inq(int ncid, int *ndimsp, int *nvarsp, int *nattsp, int *unlimdimidp)
 
    assert(h5 && grp && nc);
 
-#if 0 /*def USE_PNETCDF*/
-   /* Take care of files created/opened with parallel-netcdf library. */
-   if (h5->pnetcdf_file)
-      return ncmpi_inq(nc->int_ncid, ndimsp, nvarsp, nattsp, unlimdimidp);
-#endif /* USE_PNETCDF */
-
    /* Count the number of dims, vars, and global atts. */
    if (ndimsp)
    {
@@ -3236,9 +3244,13 @@ NC4_inq(int ncid, int *ndimsp, int *nvarsp, int *nattsp, int *unlimdimidp)
    }
    if (nvarsp)
    {
+      int i;
       *nvarsp = 0;
-      for (var = grp->var; var; var= var->l.next)
-	(*nvarsp)++;
+      for (i=0; i < grp->vars.nelems; i++)
+      {
+	if (grp->vars.value[i])
+	  (*nvarsp)++;
+      }
    }
    if (nattsp)
      {
@@ -3267,36 +3279,6 @@ NC4_inq(int ncid, int *ndimsp, int *nvarsp, int *nattsp, int *unlimdimidp)
    return NC_NOERR;
 }
 
-#if 0
-int
-NC4_set_content(int ncid, size_t size, void* memory)
-{
-    int retval = NC_NOERR;
-    herr_t herr;
-    NC *nc;
-    NC_HDF5_FILE_INFO_T *h5;
-    NC_GRP_INFO_T *grp;
-
-    LOG((4,"%s: ncid 0x%x size %ld memory 0x%x", __func__, ncid, size, memory));
-
-    /* Find file metadata. */
-    if ((retval = nc4_find_nc_grp_h5(ncid, &nc, &grp, &h5)))
-	return retval;
-    assert(h5 && grp && nc);
-
-#ifdef USE_DISKLESS
-    herr = H5Pset_file_image(h5->hdfid,memory,size);
-    if(herr)
-	BAIL(NC_EHDFERR);
-#else
-    retval = NC_EDISKLESS;
-#endif    				
-
-done:
-    return retval;
-}
-#endif
-
 /* This function will do the enddef stuff for a netcdf-4 file. */
 int
 nc4_enddef_netcdf4_file(NC_HDF5_FILE_INFO_T *h5)
@@ -3335,3 +3317,24 @@ nc_use_parallel_enabled()
    return 0;
 }
 #endif /* USE_PARALLEL4 */
+
+
+/*
+Wrap HDF5 allocated memory free operations
+*/
+
+static void
+hdf5free(void* memory)
+{
+#ifndef JNA
+    /* On Windows using the microsoft runtime, it is an error
+       for one library to free memory allocated by a different library.*/
+#ifdef HDF5_HAS_H5FREE
+    if(memory != NULL) H5free_memory(memory);
+#else
+#ifndef _MSC_VER
+	if(memory != NULL) free(memory);
+#endif
+#endif
+#endif
+}
