@@ -3,7 +3,7 @@
 Copyright 2011 University Corporation for Atmospheric
 Research/Unidata. See \ref copyright file for more info.  */
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
@@ -33,8 +33,9 @@ int optind;
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif	/* HAVE_LOCALE_H */
-#include "netcdf.h"
+
 #include "netcdf_mem.h"
+#include "netcdf.h"
 #include "utils.h"
 #include "nccomps.h"
 #include "nctime0.h"		/* new iso time and calendar stuff */
@@ -44,6 +45,8 @@ int optind;
 #include "indent.h"
 #include "isnan.h"
 #include "cdl.h"
+#include "nclog.h"
+#include "ncwinpath.h"
 
 #ifdef USE_NETCDF4
 #include "nc4internal.h" /* to get name of the special properties file */
@@ -162,10 +165,7 @@ name_path(const char *path)
     /* See if this is a url */
     {
 	char* base;
-
         extern int nc__testurl(const char*,char**);
-
-
  	if(nc__testurl(path,&base)) {
  	    return base; /* Looks like a url */
 	}
@@ -340,7 +340,7 @@ fileopen(const char* path, void** memp, size_t* sizep)
 #ifdef vms
     fd = open(path, oflags, 0, "ctx=stm");
 #else
-    fd  = open(path, oflags);
+    fd  = NCopen2(path, oflags);
 #endif
     if(fd < 0) {
 	status = errno;
@@ -1261,6 +1261,7 @@ print_enum_type(int ncid, nc_type typeid) {
     char safe_buf[SAFE_BUF_LEN];
     char *delim;
     int64_t data;	    /* space for data of any primitive type */
+    void* raw;
     char *esc_btn;
     char *esc_tn;
     char *esc_mn;
@@ -1283,30 +1284,31 @@ print_enum_type(int ncid, nc_type typeid) {
 	if (f == type_nfields - 1)
 	    delim = "} ;\n";
 	NC_CHECK( nc_inq_enum_member(ncid, typeid, f, memname, &data) );
+	raw = (void*)&data;
 	switch (base_nc_type) {
 	case NC_BYTE:
-	    memval = *(char *)&data;
+	    memval = *(char *)raw;
 	    break;
 	case NC_SHORT:
-	    memval = *(short *)&data;
+	    memval = *(short *)raw;
 	    break;
 	case NC_INT:
-	    memval = *(int *)&data;
+	    memval = *(int *)raw;
 	    break;
 	case NC_UBYTE:
-	    memval = *(unsigned char *)&data;
+	    memval = *(unsigned char *)raw;
 	    break;
 	case NC_USHORT:
-	    memval = *(unsigned short *)&data;
+	    memval = *(unsigned short *)raw;
 	    break;
 	case NC_UINT:
-	    memval = *(unsigned int *)&data;
+	    memval = *(unsigned int *)raw;
 	    break;
 	case NC_INT64:
-	    memval = *(int64_t *)&data;
+	    memval = *(int64_t *)raw;
 	    break;
 	case NC_UINT64:
-	    memval = *(uint64_t *)&data;
+	    memval = *(uint64_t *)raw;
 	    break;
 	default:
 	    error("Bad base type for enum!");
@@ -2228,6 +2230,7 @@ main(int argc, char *argv[])
 	    nc_set_log_level(level);
 	  }
 #endif
+	  ncsetlogging(1);
 	  break;
         case '?':
 	  usage();
@@ -2299,9 +2302,9 @@ main(int argc, char *argv[])
                                              &formatting_specs.nc_extended,
                                              &formatting_specs.nc_mode) );
 	    if (kind_out) {
-		printf ("%s\n", kind_string(formatting_specs.nc_kind));
+		printf ("%s", kind_string(formatting_specs.nc_kind));
 	    } else if (kind_out_extended) {
-		printf ("%s\n", kind_string_extended(formatting_specs.nc_extended,formatting_specs.nc_mode));
+		printf ("%s", kind_string_extended(formatting_specs.nc_extended,formatting_specs.nc_mode));
 	    } else {
 		/* Initialize list of types. */
 		init_types(ncid);
