@@ -1,5 +1,8 @@
 #!/bin/sh
-if test "x$SETX" = x1 ; then echo "file=$0"; set -x ; fi
+
+if test "x$srcdir" = x ; then srcdir=`pwd`; fi 
+. ../test_common.sh
+
 # This shell script runs ncdump to verify scalar attribute and variable output
 
 set -e
@@ -7,7 +10,7 @@ echo ""
 echo "*** Running ncdump scalar test."
 
 # echo "*** dumping tst_h_scalar.nc to tst_h_scalar.cdl..."
-./ncdump tst_h_scalar.nc > tst_h_scalar.cdl
+${NCDUMP} tst_h_scalar.nc > tst_h_scalar.cdl
 # echo "*** comparing tst_h_scalar.cdl with ref_tst_h_scalar.cdl..."
 diff -b tst_h_scalar.cdl $srcdir/cdl/ref_tst_h_scalar.cdl
 
