@@ -4,7 +4,7 @@
  *   $Header: /upc/share/CVS/netcdf-3/ncdump/vardata.c,v 1.48 2010/05/05 22:15:39 dmh Exp $
  *********************************************************************/
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -64,7 +64,7 @@ lput(const char *cp) {
 	linep = (int)strlen(LINEPIND) + indent_get();
     }
     (void) fputs(cp,stdout);
-    if (cp[nn - 1] == '\n') {
+    if (nn > 0 && cp[nn - 1] == '\n') {
 	linep = indent_get();
     } else
 	linep += nn;
