@@ -10,6 +10,8 @@
 #ifndef NCGEN_INCLUDES_H
 #define NCGEN_INCLUDES_H
 
+#undef USE_NOFILL
+
 #include "config.h"
 
 #include <stdlib.h>
@@ -31,6 +33,7 @@
 
 #include "list.h"
 #include "bytebuffer.h"
+#include "nctime.h"
 
 /* Local Configuration flags*/
 #define ENABLE_BINARY
@@ -39,19 +42,20 @@
 #define ENABLE_JAVA
 
 #include "netcdf.h"
-#include "odom.h"
 #include "data.h"
 #include "ncgen.h"
 #include "genlib.h"
 #include "util.h"
 #include "debug.h"
+#include "nc.h"
 
-#ifdef HAVE_STRDUP
-#define nulldup(s) ((s)==NULL?NULL:strdup(s))
-#else
-extern char* nulldup(const char*);
+#ifdef USE_NETCDF4
+#include "nc4internal.h"
 #endif
 
 extern int specialconstants;
 
+#undef ITERBUG
+#undef CHARBUG
+
 #endif /* NCGEN_INCLUDES_H */
