@@ -6,7 +6,8 @@
 /* $Header: /upc/share/CVS/netcdf-3/ncgen/main.c,v 1.33 2010/05/26 21:43:36 dmh Exp $ */
 
 #include "includes.h"
-#include "offsets.h"
+#include "ncoffsets.h"
+#include "ncwinpath.h"
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
@@ -453,7 +454,7 @@ main(
     if (argc > 0 && strcmp(argv[0], "-") != 0) {
 	char bom[4];
 	size_t count;
-	if ((fp = fopen(argv[0], "r")) == NULL) {
+	if ((fp = NCfopen(argv[0], "r")) == NULL) {
 	    derror ("can't open file %s for reading: ", argv[0]);
 	    perror("");
 	    return(7);
@@ -497,6 +498,13 @@ main(
 
     /* Compute the k_flag (1st pass) using rules in the man page (ncgen.1).*/
 
+#ifndef USE_CDF5
+    if(k_flag == NC_FORMAT_CDF5) {
+      derror("Output format CDF5 requested, but netcdf was built without cdf5 support.");
+      return 0;
+    }
+#endif
+
 #ifndef USE_NETCDF4
     if(enhanced_flag) {
 	derror("CDL input is enhanced mode, but --disable-netcdf4 was specified during build");
@@ -505,7 +513,7 @@ main(
 #endif
 
     if(l_flag == L_JAVA || l_flag == L_F77) {
-        k_flag = 1;
+        k_flag = NC_FORMAT_CLASSIC;
 	if(enhanced_flag) {
 	    derror("Java or Fortran requires classic model CDL input");
 	    return 0;
@@ -516,12 +524,12 @@ main(
       k_flag = globalspecials._Format;
 
     if(cdf5_flag && !enhanced_flag && k_flag == 0)
-      k_flag = 5;
+      k_flag = NC_FORMAT_64BIT_DATA;
     if(enhanced_flag && k_flag == 0)
-      k_flag = 3;
+      k_flag = NC_FORMAT_NETCDF4;
 
-    if(enhanced_flag && k_flag != 3) {
-      if(enhanced_flag && k_flag != 3 && k_flag != 5) {
+    if(enhanced_flag && k_flag != NC_FORMAT_NETCDF4) {
+      if(enhanced_flag && k_flag != NC_FORMAT_NETCDF4 && k_flag != NC_FORMAT_64BIT_DATA) {
         derror("-k or _Format conflicts with enhanced CDL input");
         return 0;
       }
@@ -529,13 +537,13 @@ main(
 
     if(specials_flag > 0 && k_flag == 0)
 #ifdef USE_NETCDF4
-	k_flag = 3;
+	k_flag = NC_FORMAT_NETCDF4;
 #else
-	k_flag = 1;
+	k_flag = NC_FORMAT_CLASSIC;
 #endif
 
     if(k_flag == 0)
-	k_flag = 1;
+	k_flag = NC_FORMAT_CLASSIC;
 
     /* Figure out usingclassic */
     switch (k_flag) {
@@ -580,7 +588,6 @@ END_OF_MAIN()
 void
 init_netcdf(void) /* initialize global counts, flags */
 {
-    compute_alignments();
     memset((void*)&nullconstant,0,sizeof(NCConstant));
     fillconstant = nullconstant;
     fillconstant.nctype = NC_FILLVALUE;
