@@ -3,10 +3,10 @@
 */
 
 #include "config.h"
-#include <sys/stat.h>
 
-#ifdef NETINET_IN_H
-#include <netinet/in.h>
+#include <stdlib.h>
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
 #endif
 
 #include "ocinternal.h"
@@ -39,19 +39,19 @@ static char*
 dent2(int n) {return dent(n+4);}
 
 static void
-tabto(int pos, OCbytes* buffer)
+tabto(int pos, NCbytes* buffer)
 {
     int bol,len,pad;
-    len = ocbyteslength(buffer);
+    len = ncbyteslength(buffer);
     /* find preceding newline */
     for(bol=len-1;;bol--) {
-	int c = ocbytesget(buffer,bol);
+	int c = ncbytesget(buffer,(size_t)bol);
 	if(c < 0) break;
 	if(c == '\n') {bol++; break;}
     }
     len = (len - bol);
     pad = (pos - len);
-    while(pad-- > 0) ocbytescat(buffer," ");
+    while(pad-- > 0) ncbytescat(buffer," ");
 }
 
 void
@@ -83,8 +83,8 @@ dumpocnode1(OCnode* node, int depth)
         fprintf(stdout,"[%2d]%s ",depth,dent(depth));
 	fprintf(stdout,"dataset %s\n",
 		(node->name?node->name:""));
-	for(n=0;n<oclistlength(node->subnodes);n++) {
-	    dumpocnode1((OCnode*)oclistget(node->subnodes,n),depth+1);
+	for(n=0;n<nclistlength(node->subnodes);n++) {
+	    dumpocnode1((OCnode*)nclistget(node->subnodes,n),depth+1);
 	}
     } break;
 
@@ -95,8 +95,8 @@ dumpocnode1(OCnode* node, int depth)
 	dumpdimensions(node);
 	fprintf(stdout," &%lx",(unsigned long)node);
 	fprintf(stdout,"\n");
-	for(n=0;n<oclistlength(node->subnodes);n++) {
-	    dumpocnode1((OCnode*)oclistget(node->subnodes,n),depth+1);
+	for(n=0;n<nclistlength(node->subnodes);n++) {
+	    dumpocnode1((OCnode*)nclistget(node->subnodes,n),depth+1);
 	}
     } break;
 
@@ -107,8 +107,8 @@ dumpocnode1(OCnode* node, int depth)
 	dumpdimensions(node);
 	fprintf(stdout," &%lx",(unsigned long)node);
 	fprintf(stdout,"\n");
-	for(n=0;n<oclistlength(node->subnodes);n++) {
-	    dumpocnode1((OCnode*)oclistget(node->subnodes,n),depth+1);
+	for(n=0;n<nclistlength(node->subnodes);n++) {
+	    dumpocnode1((OCnode*)nclistget(node->subnodes,n),depth+1);
 	}
     } break;
 
@@ -121,10 +121,10 @@ dumpocnode1(OCnode* node, int depth)
 	fprintf(stdout," &%lx",(unsigned long)node);
 	fprintf(stdout,"\n");
 	fprintf(stdout,"%sarray:\n",dent2(depth+1));
-	dumpocnode1((OCnode*)oclistget(node->subnodes,0),depth+2);
+	dumpocnode1((OCnode*)nclistget(node->subnodes,0),depth+2);
 	fprintf(stdout,"%smaps:\n",dent2(depth+1));
-	for(i=1;i<oclistlength(node->subnodes);i++) {
-	    dumpocnode1((OCnode*)oclistget(node->subnodes,i),depth+2);
+	for(i=1;i<nclistlength(node->subnodes);i++) {
+	    dumpocnode1((OCnode*)nclistget(node->subnodes,i),depth+2);
 	}
     } break;
 
@@ -132,8 +132,8 @@ dumpocnode1(OCnode* node, int depth)
         fprintf(stdout,"[%2d]%s ",depth,dent(depth));
 	if(node->name == NULL) OCPANIC("Attribute without name");
 	fprintf(stdout,"%s %s",octypetostring(node->etype),node->name);
-	for(n=0;n<oclistlength(node->att.values);n++) {
-	    char* value = (char*)oclistget(node->att.values,n);
+	for(n=0;n<nclistlength(node->att.values);n++) {
+	    char* value = (char*)nclistget(node->att.values,n);
 	    if(n > 0) fprintf(stdout,",");
 	    fprintf(stdout," %s",value);
 	}
@@ -144,8 +144,8 @@ dumpocnode1(OCnode* node, int depth)
     case OC_Attributeset: {
         fprintf(stdout,"[%2d]%s ",depth,dent(depth));
 	fprintf(stdout,"%s:\n",node->name?node->name:"Attributes");
-	for(n=0;n<oclistlength(node->subnodes);n++) {
-	    dumpocnode1((OCnode*)oclistget(node->subnodes,n),depth+1);
+	for(n=0;n<nclistlength(node->subnodes);n++) {
+	    dumpocnode1((OCnode*)nclistget(node->subnodes,n),depth+1);
 	}
     } break;
 
@@ -155,15 +155,15 @@ dumpocnode1(OCnode* node, int depth)
 
     if(node->attributes != NULL) {
 	unsigned int i;
-	for(i=0;i<oclistlength(node->attributes);i++) {
-	    OCattribute* att = (OCattribute*)oclistget(node->attributes,i);
+	for(i=0;i<nclistlength(node->attributes);i++) {
+	    OCattribute* att = (OCattribute*)nclistget(node->attributes,i);
 	    fprintf(stdout,"%s[%s=",dent2(depth+2),att->name);
 	    if(att->nvalues == 0)
 		OCPANIC("Attribute.nvalues == 0");
 	    if(att->nvalues == 1) {
 		dumpattvalue(att->etype,att->values,0);
 	    } else {
-		unsigned int j;
+		int j;
 	        fprintf(stdout,"{");
 		for(j=0;j<att->nvalues;j++) {
 		    if(j>0) fprintf(stdout,", ");
@@ -181,7 +181,7 @@ dumpdimensions(OCnode* node)
 {
     unsigned int i;
     for(i=0;i<node->array.rank;i++) {
-        OCnode* dim = (OCnode*)oclistget(node->array.dimensions,i);
+        OCnode* dim = (OCnode*)nclistget(node->array.dimensions,i);
         fprintf(stdout,"[%s=%lu]",
 			(dim->name?dim->name:"?"),
 			(unsigned long)dim->dim.declsize);
@@ -212,18 +212,18 @@ void
 ocdumpclause(OCprojectionclause* ref)
 {
     unsigned int i;
-    OClist* path = oclistnew();
+    NClist* path = nclistnew();
     occollectpathtonode(ref->node,path);
-    for(i=0;i<oclistlength(path);i++) {
-        OClist* sliceset;
-	OCnode* node = (OCnode*)oclistget(path,i);
+    for(i=0;i<nclistlength(path);i++) {
+        NClist* sliceset;
+	OCnode* node = (OCnode*)nclistget(path,i);
 	if(node->tree != NULL) continue; /* leave off the root node*/
 	fprintf(stdout,"%s%s",(i>0?PATHSEPARATOR:""),node->name);
-	sliceset = (OClist*)oclistget(ref->indexsets,i);
+	sliceset = (NClist*)nclistget(ref->indexsets,i);
 	if(sliceset != NULL) {
 	    unsigned int j;
-	    for(j=0;j<oclistlength(sliceset);j++) {
-	        OCslice* slice = (OCslice*)oclistget(sliceset,j);
+	    for(j=0;j<nclistlength(sliceset);j++) {
+	        OCslice* slice = (OCslice*)nclistget(sliceset,j);
 	        ocdumpslice(slice);
 	    }
 	}
@@ -243,7 +243,7 @@ addfield(char* field, char* line, int align)
 }
 
 static void
-dumpfield(int index, char* n8, int isxdr)
+dumpfield(size_t index, char* n8, int isxdr)
 {
     char line[1024];
     char tmp[32];
@@ -263,7 +263,7 @@ dumpfield(int index, char* n8, int isxdr)
     line[0] = '\0';
 
     /* offset */
-    sprintf(tmp,"%6d",index);
+    sprintf(tmp,"%6zd",index);
     addfield(tmp,line,5);
 
     memcpy(form.cv,n8,4);
@@ -306,7 +306,7 @@ dumpfield(int index, char* n8, int isxdr)
     addfield(tmp,line,16);
 
     /* double */
-    memcpy(dform.cv,n8,2*XDRUNIT);
+    memcpy(dform.cv,n8,(size_t)(2*XDRUNIT));
     if(isxdr) xxdrntohdouble(dform.cv,&dform.d);
     sprintf(tmp,"%#g",dform.d);
     addfield(tmp,line,12);
@@ -409,36 +409,61 @@ ocdumpmemory(char* memory, size_t len, int xdrencoded, int level)
     }
 }
 
-static int
-ocreadfile(FILE* file, int datastart, char** memp, size_t* lenp)
+static OCerror
+ocreadfile(FILE* file, off_t datastart, char** memp, size_t* lenp)
 {
-    char* mem;
+    char* mem = NULL;
     size_t len;
-    ssize_t pos;
-    size_t red;
+    long red;
     struct stat stats;
+    long pos;
+    OCerror stat = OC_NOERR;
 
     pos = ftell(file);
-    fseek(file,0,SEEK_SET);
-    fseek(file,datastart,SEEK_SET);
+    if(pos < 0) {
+      fprintf(stderr,"ocreadfile: ftell error.\n");
+      stat = OC_ERCFILE;
+      goto done;
+    }
 
-    fstat(fileno(file),&stats);
+    fseek(file,0,SEEK_SET);
+    if(fseek(file,(long)datastart,SEEK_SET) < 0) {
+	fprintf(stderr,"ocreadfile: fseek error.\n");
+	stat = OC_ERCFILE;
+	goto done;
+    }
+
+    if(fstat(fileno(file),&stats) < 0) {
+	fprintf(stderr,"ocreadfile: fstat error.\n");
+	stat = OC_ERCFILE;
+	goto done;
+    }
     len = stats.st_size;
     len -= datastart;
-    
+
     mem = (char*)calloc(len+1,1);
-    if(mem == NULL) return 0;
+    if(mem == NULL) {stat = OC_ENOMEM; goto done;}
 
     /* Read only the data part */
     red = fread(mem,1,len,file);
     if(red < len) {
 	fprintf(stderr,"ocreadfile: short file\n");
-	return 0;
-    }	
-    fseek(file,pos,SEEK_SET); /* leave it as we found it*/
-    if(memp) *memp = mem;
+	stat = OC_ERCFILE;
+	goto done;
+    }
+
+    if(fseek(file,pos,SEEK_SET) < 0) {; /* leave it as we found it*/
+      fprintf(stderr,"ocreadfile: fseek error.\n");
+      stat = OC_ERCFILE;
+      goto done;
+    }
+    if(memp) {*memp = mem; mem = NULL;}
     if(lenp) *lenp = len;
-    return 1;
+
+ done:
+    if(mem != NULL)
+      free(mem);
+    return OCTHROW(stat);
 }
 
 void
@@ -447,10 +472,15 @@ ocdd(OCstate* state, OCnode* root, int xdrencoded, int level)
     char* mem;
     size_t len;
     if(root->tree->data.file != NULL) {
-        if(!ocreadfile(root->tree->data.file,root->tree->data.bod,&mem,&len)) {
-    	    fprintf(stderr,"ocdd could not read data file\n");
-	    return;
-	}
+        if(!ocreadfile(root->tree->data.file,
+                       root->tree->data.bod,
+                       &mem,
+                       &len)) {
+          /* ocreadfile allocates memory that must be freed. */
+          if(mem != NULL) free(mem);
+          fprintf(stderr,"ocdd could not read data file\n");
+          return;
+        }
         ocdumpmemory(mem,len,xdrencoded,level);
         free(mem);
     } else {
@@ -463,34 +493,34 @@ ocdd(OCstate* state, OCnode* root, int xdrencoded, int level)
 }
 
 void
-ocdumpdata(OCstate* state, OCdata* data, OCbytes* buffer, int frominstance)
+ocdumpdata(OCstate* state, OCdata* data, NCbytes* buffer, int frominstance)
 {
     char tmp[1024];
-    OCnode* template = data->template;
+    OCnode* pattern = data->pattern;
     snprintf(tmp,sizeof(tmp),"%lx:",(unsigned long)data);
-    ocbytescat(buffer,tmp);
+    ncbytescat(buffer,tmp);
     if(!frominstance) {
-        ocbytescat(buffer," node=");
-        ocbytescat(buffer,template->name);
+        ncbytescat(buffer," node=");
+        ncbytescat(buffer,pattern->name);
     }
     snprintf(tmp,sizeof(tmp)," xdroffset=%ld",(unsigned long)data->xdroffset);
-    ocbytescat(buffer,tmp);
-    if(data->template->octype == OC_Atomic) {
+    ncbytescat(buffer,tmp);
+    if(data->pattern->octype == OC_Atomic) {
         snprintf(tmp,sizeof(tmp)," xdrsize=%ld",(unsigned long)data->xdrsize);
-        ocbytescat(buffer,tmp);
+        ncbytescat(buffer,tmp);
     }
-    if(iscontainer(template->octype)) {
+    if(ociscontainer(pattern->octype)) {
         snprintf(tmp,sizeof(tmp)," ninstances=%d",(int)data->ninstances);
-        ocbytescat(buffer,tmp);
-    } else if(template->etype == OC_String || template->etype == OC_URL) {
+        ncbytescat(buffer,tmp);
+    } else if(pattern->etype == OC_String || pattern->etype == OC_URL) {
         snprintf(tmp,sizeof(tmp)," nstrings=%d",(int)data->nstrings);
-        ocbytescat(buffer,tmp);
+        ncbytescat(buffer,tmp);
     }
-    ocbytescat(buffer," container=");
+    ncbytescat(buffer," container=");
     snprintf(tmp,sizeof(tmp),"%lx",(unsigned long)data->container);
-    ocbytescat(buffer,tmp);
-    ocbytescat(buffer," mode=");
-    ocbytescat(buffer,ocdtmodestring(data->datamode,0));
+    ncbytescat(buffer,tmp);
+    ncbytescat(buffer," mode=");
+    ncbytescat(buffer,ocdtmodestring(data->datamode,0));
 }
 
 /*
@@ -504,10 +534,10 @@ static const char* header =
 "Depth Offset   Index Flags Size Type     Name\n";
 
 void
-ocdumpdatatree(OCstate* state, OCdata* data, OCbytes* buffer, int depth)
+ocdumpdatatree(OCstate* state, OCdata* data, NCbytes* buffer, int depth)
 {
     size_t i,rank;
-    OCnode* template;
+    OCnode* pattern;
     char tmp[1024];
     size_t crossproduct;
     int tabstop = 0;
@@ -515,25 +545,25 @@ ocdumpdatatree(OCstate* state, OCdata* data, OCbytes* buffer, int depth)
 
     /* If this is the first call, then dump a header line */
     if(depth == 0) {
-	ocbytescat(buffer,header);
+	ncbytescat(buffer,header);
     }
 
-    /* get info about the template */
-    template = data->template;
-    rank = template->array.rank;
+    /* get info about the pattern */
+    pattern = data->pattern;
+    rank = pattern->array.rank;
 
     /* get total dimension size */
     if(rank > 0)
-        crossproduct = octotaldimsize(template->array.rank,template->array.sizes);
+        crossproduct = octotaldimsize(pattern->array.rank,pattern->array.sizes);
 
     /* Dump the depth first */
     snprintf(tmp,sizeof(tmp),"[%03d]",depth);
-    ocbytescat(buffer,tmp);
+    ncbytescat(buffer,tmp);
 
     tabto(tabstops[++tabstop],buffer);
 
     snprintf(tmp,sizeof(tmp),"%08lu",(unsigned long)data->xdroffset);
-    ocbytescat(buffer,tmp);
+    ncbytescat(buffer,tmp);
 
     tabto(tabstops[++tabstop],buffer);
 
@@ -542,14 +572,14 @@ ocdumpdatatree(OCstate* state, OCdata* data, OCbytes* buffer, int depth)
        || fisset(data->datamode,OCDT_ELEMENT)
        || fisset(data->datamode,OCDT_RECORD)) {
         snprintf(tmp,sizeof(tmp),"%04lu ",(unsigned long)data->index);
-        ocbytescat(buffer,tmp);
+        ncbytescat(buffer,tmp);
     }
-   
+
     tabto(tabstops[++tabstop],buffer);
 
     /* Dump the mode flags in compact form */
-    ocbytescat(buffer,ocdtmodestring(data->datamode,1));
-   
+    ncbytescat(buffer,ocdtmodestring(data->datamode,1));
+
     tabto(tabstops[++tabstop],buffer);
 
     /* Dump the size or ninstances */
@@ -559,28 +589,28 @@ ocdumpdatatree(OCstate* state, OCdata* data, OCbytes* buffer, int depth)
     } else {
         snprintf(tmp,sizeof(tmp),"%04lu",(unsigned long)data->xdrsize);
     }
-    ocbytescat(buffer,tmp);
+    ncbytescat(buffer,tmp);
 
     tabto(tabstops[++tabstop],buffer);
 
-    if(template->octype == OC_Atomic) {
-	typename = octypetoddsstring(template->etype);
+    if(pattern->octype == OC_Atomic) {
+	typename = octypetoddsstring(pattern->etype);
     } else { /*must be container*/
-	typename = octypetoddsstring(template->octype);
+	typename = octypetoddsstring(pattern->octype);
     }
-    ocbytescat(buffer,typename);
+    ncbytescat(buffer,typename);
 
     tabto(tabstops[++tabstop],buffer);
 
-    if(!occopycat(tmp,sizeof(tmp),1,template->name))
+    if(!occopycat(tmp,sizeof(tmp),1,pattern->name))
 	return;
-    ocbytescat(buffer,tmp);
+    ncbytescat(buffer,tmp);
 
     if(rank > 0) {
 	snprintf(tmp,sizeof(tmp),"[%lu]",(unsigned long)crossproduct);
-	ocbytescat(buffer,tmp);
+	ncbytescat(buffer,tmp);
     }
-    ocbytescat(buffer,"\n");
+    ncbytescat(buffer,"\n");
 
     /* dump the sub-instance, which might be fields, records, or elements */
     if(!fisset(data->datamode,OCDT_ATOMIC)) {
@@ -590,13 +620,13 @@ ocdumpdatatree(OCstate* state, OCdata* data, OCbytes* buffer, int depth)
 }
 
 void
-ocdumpdatapath(OCstate* state, OCdata* data, OCbytes* buffer)
+ocdumpdatapath(OCstate* state, OCdata* data, NCbytes* buffer)
 {
     int i;
     OCdata* path[1024];
     char tmp[1024];
     OCdata* pathdata;
-    OCnode* template;
+    OCnode* pattern;
     int isrecord;
 
     path[0] = data;
@@ -604,13 +634,13 @@ ocdumpdatapath(OCstate* state, OCdata* data, OCbytes* buffer)
 	OCdata* next = path[i-1];
 	if(next->container == NULL) break;
 	path[i] = next->container;
-    }    	        
+    }
     /* Path is in reverse order */
     for(i=i-1;i>=0;i--) {
 	pathdata = path[i];
-	template = pathdata->template;
-	ocbytescat(buffer,"/");
-	ocbytescat(buffer,template->name);
+	pattern = pathdata->pattern;
+	ncbytescat(buffer,"/");
+	ncbytescat(buffer,pattern->name);
 	/* Check the mode of the next step in path */
 	if(i > 0) {
 	    OCdata* next = path[i-1];
@@ -618,35 +648,35 @@ ocdumpdatapath(OCstate* state, OCdata* data, OCbytes* buffer)
 		|| fisset(next->datamode,OCDT_ELEMENT)
 		|| fisset(next->datamode,OCDT_RECORD)) {
 		snprintf(tmp,sizeof(tmp),".%lu",(unsigned long)next->index);
-	        ocbytescat(buffer,tmp);
+	        ncbytescat(buffer,tmp);
 	    }
 	}
-	if(template->octype == OC_Atomic) {
-	    if(template->array.rank > 0) {
-	        off_t xproduct = octotaldimsize(template->array.rank,template->array.sizes);
+	if(pattern->octype == OC_Atomic) {
+	    if(pattern->array.rank > 0) {
+	        off_t xproduct = octotaldimsize(pattern->array.rank,pattern->array.sizes);
 	        snprintf(tmp,sizeof(tmp),"[0..%lu]",(unsigned long)xproduct-1);
-	        ocbytescat(buffer,tmp);
+	        ncbytescat(buffer,tmp);
 	    }
 	}
 	isrecord = 0;
-	if(template->octype == OC_Sequence) {
+	if(pattern->octype == OC_Sequence) {
 	    /* Is this a record or a sequence ? */
 	    isrecord = (fisset(pathdata->datamode,OCDT_RECORD) ? 1 : 0);
 	}
     }
     /* Add suffix to path */
-    if(iscontainer(template->octype)) {
+    if(ociscontainer(pattern->octype)) {
         /* add the container type, except distinguish record and sequence */
-	ocbytescat(buffer,":");
+	ncbytescat(buffer,":");
 	if(isrecord)
-	    ocbytescat(buffer,"Record");
+	    ncbytescat(buffer,"Record");
 	else
-	    ocbytescat(buffer,octypetoddsstring(template->octype));
-    } else if(isatomic(template->octype)) {
+	    ncbytescat(buffer,octypetoddsstring(pattern->octype));
+    } else if(ocisatomic(pattern->octype)) {
 	/* add the atomic etype */
-	ocbytescat(buffer,":");
-	ocbytescat(buffer,octypetoddsstring(template->etype));
+	ncbytescat(buffer,":");
+	ncbytescat(buffer,octypetoddsstring(pattern->etype));
     }
     snprintf(tmp,sizeof(tmp),"->0x%0lx",(unsigned long)pathdata);
-    ocbytescat(buffer,tmp);	
+    ncbytescat(buffer,tmp);
 }
