@@ -3,144 +3,159 @@
 
 #include "config.h"
 #include <stdio.h>
-#include <fcntl.h>
-#include <errno.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+#if _MSC_VER
+#include <io.h>
+#include <direct.h>
+#include <process.h>
+#endif
+
+#ifdef HAVE_FTW_H
+#include <ftw.h>
+#endif
+
+#include <errno.h>
 
 #include "ocinternal.h"
 #include "ocdebug.h"
 #include "occlientparams.h"
-#include "ocrc.h"
 #include "occurlfunctions.h"
 #include "ochttp.h"
 #include "ocread.h"
 #include "dapparselex.h"
+#include "ncwinpath.h"
 
-/* Note: TMPPATH must end in '/' */
-#ifdef __CYGWIN__
-#define TMPPATH1 "/cygdrive/c/temp/"
-#define TMPPATH2 "./"
-#elif _WIN32
-#define TMPPATH1 "c:\\temp\\"
-#define TMPPATH2 ".\\"
-#else
-#define TMPPATH1 "/tmp/"
-#define TMPPATH2 "./"
-#endif
+#define DATADDSFILE "datadds"
 
 #define CLBRACE '{'
 #define CRBRACE '}'
 
-
-/* Define default rc files and aliases*/
-static char* rcfilenames[4] = {".daprc",".dodsrc",".ocrc",NULL};
-
-static int ocextractddsinmemory(OCstate*,OCtree*,int);
-static int ocextractddsinfile(OCstate*,OCtree*,int);
+/*Forward*/
+static OCerror ocextractddsinmemory(OCstate*,OCtree*,int);
+static OCerror ocextractddsinfile(OCstate*,OCtree*,int);
 static char* constraintescape(const char* url);
 static OCerror createtempfile(OCstate*,OCtree*);
-static int createtempfile1(char*,char**);
 static int dataError(XXDR* xdrs, OCstate*);
+static void ocremovefile(const char* path);
 
-static void ocsetcurlproperties(OCstate*);
+static OCerror ocset_curlproperties(OCstate*);
 
 extern OCnode* makeunlimiteddimension(void);
 
-#ifdef _WIN32
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-#define _S_IREAD 256
-#define _S_IWRITE 128
-#else
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-#endif
-
 /* Collect global state info in one place */
 struct OCGLOBALSTATE ocglobalstate;
 
-int
+OCerror
 ocinternalinitialize(void)
 {
     int stat = OC_NOERR;
 
+#if 0
+    if(sizeof(off_t) != sizeof(void*)) {
+      fprintf(stderr,"OC xxdr depends on the assumption that sizeof(off_t) == sizeof(void*)\n");
+      /*
+	Commenting out for now, as this does not hold true on 32-bit linux systems.
+      OCASSERT(sizeof(off_t) == sizeof(void*));
+	*/
+    }
+#endif
+
     if(!ocglobalstate.initialized) {
-        memset((void*)&ocglobalstate,0,sizeof(ocglobalstate));
-	ocglobalstate.initialized = 1;
+      CURLcode cstat = CURLE_OK;
+      cstat = curl_global_init(CURL_GLOBAL_ALL);
+      if(cstat != CURLE_OK)
+	fprintf(stderr,"curl_global_init failed!\n");
+      memset((void*)&ocglobalstate,0,sizeof(ocglobalstate));
+      ocglobalstate.initialized = 1;
+    }
+
+    /* Capture temp dir*/
+    {
+	char* tempdir;
+	char* p;
+	char* q;
+	char cwd[4096];
+#ifdef _MSC_VER
+        tempdir = getenv("TEMP");
+#else
+	tempdir = "/tmp";
+#endif
+        if(tempdir == NULL) {
+	    fprintf(stderr,"Cannot find a temp dir; using ./\n");
+	    tempdir = getcwd(cwd,sizeof(cwd));
+	    if(tempdir == NULL || *tempdir == '\0') tempdir = ".";
+	}
+        ocglobalstate.tempdir= (char*)malloc(strlen(tempdir) + 1);
+	for(p=tempdir,q=ocglobalstate.tempdir;*p;p++,q++) {
+	    if((*p == '/' && *(p+1) == '/')
+	       || (*p == '\\' && *(p+1) == '\\')) {p++;}
+	    *q = *p;
+	}
+	*q = '\0';
+#ifdef _MSC_VER
+#else
+        /* Canonicalize */
+	for(p=ocglobalstate.tempdir;*p;p++) {
+	    if(*p == '\\') {*p = '/'; };
+	}
+	*q = '\0';
+#endif
+    }
+
+    /* Capture $HOME */
+    {
+	char* p;
+	char* q;
+        char* home = getenv("HOME");
+
+        if(home == NULL) {
+	    /* use tempdir */
+	    home = ocglobalstate.tempdir;
+	}
+        ocglobalstate.home = (char*)malloc(strlen(home) + 1);
+	for(p=home,q=ocglobalstate.home;*p;p++,q++) {
+	    if((*p == '/' && *(p+1) == '/')
+	       || (*p == '\\' && *(p+1) == '\\')) {p++;}
+	    *q = *p;
+	}
+	*q = '\0';
+#ifdef _MSC_VER
+#else
+        /* Canonicalize */
+	for(p=home;*p;p++) {
+	    if(*p == '\\') {*p = '/'; };
+	}
+#endif
     }
 
     /* Compute some xdr related flags */
     xxdr_init();
 
-    ocloginit();
+    ncloginit();
 
     oc_curl_protocols(&ocglobalstate); /* see what protocols are supported */
 
-    /* compile the .dodsrc, if any */
-    {
-        char* path = NULL;
-        char* homepath = NULL;
-	char** alias;
-	FILE* f = NULL;
-        /* locate the configuration files: . first in '.',  then $HOME */
-	for(alias=rcfilenames;*alias;alias++) {
-	    size_t pathlen = strlen("./")+strlen(*alias)+1;
-            path = (char*)malloc(pathlen);
-	    if(path == NULL) return OC_ENOMEM;
-	    if(!occopycat(path,pathlen,2,"./",*alias))
-		return OC_EOVERRUN;
-  	    /* see if file is readable */
-	    f = fopen(path,"r");
-	    if(f != NULL) break;
-    	    if(path != NULL) {free(path); path = NULL;} /* cleanup */
-	}
-	if(f == NULL) { /* try $HOME */
-	    OCASSERT(path == NULL);
-            homepath = getenv("HOME");
-            if (homepath!= NULL) {
-	        for(alias=rcfilenames;*alias;alias++) {
-	            size_t pathlen = strlen(homepath)+1+strlen(*alias)+1;
-	            path = (char*)malloc(pathlen);
-	            if(path == NULL) return OC_ENOMEM;
-		    if(!occopycat(path,pathlen,3,homepath,"/",*alias))
-			return OC_EOVERRUN;
-		    f = fopen(path,"r");
-		    if(f != NULL) break;
- 	            if(path != NULL) {free(path); path=NULL;}
-		}
-            }
-	}
-        if(f == NULL) {
-            oclog(OCLOGDBG,"Cannot find runtime configuration file");
-	} else {
-	    OCASSERT(path != NULL);
-       	    fclose(f);
-            if(ocdebug > 1)
-		fprintf(stderr, "DODS RC file: %s\n", path);
-            if(ocdodsrc_read(*alias,path) == 0)
-	        oclog(OCLOGERR, "Error parsing %s\n",path);
-        }
-        if(path != NULL) free(path);
-    }
     return OCTHROW(stat);
 }
 
+
 /**************************************************/
 OCerror
 ocopen(OCstate** statep, const char* url)
 {
     int stat = OC_NOERR;
     OCstate * state = NULL;
-    OCURI* tmpurl = NULL;
+    NCURI* tmpurl = NULL;
     CURL* curl = NULL; /* curl handle*/
 
-    if(!ocuriparse(url,&tmpurl)) {OCTHROWCHK(stat=OC_EBADURL); goto fail;}
-    
+    if(ncuriparse(url,&tmpurl) != NCU_OK) {OCTHROWCHK(stat=OC_EBADURL); goto fail;}
+
     stat = occurlopen(&curl);
     if(stat != OC_NOERR) {OCTHROWCHK(stat); goto fail;}
 
@@ -151,22 +166,30 @@ ocopen(OCstate** statep, const char* url)
     state->header.magic = OCMAGIC;
     state->header.occlass = OC_State;
     state->curl = curl;
-    state->trees = oclistnew();
+    state->trees = nclistnew();
     state->uri = tmpurl;
-    if(!ocuridecodeparams(state->uri)) {
-	oclog(OCLOGWARN,"Could not parse client parameters");
-    }
-    state->packet = ocbytesnew();
-    ocbytessetalloc(state->packet,DFALTPACKETSIZE); /*initial reasonable size*/
 
-    /* set curl properties for this link */
-    ocsetcurlproperties(state);
+    state->packet = ncbytesnew();
+    ncbytessetalloc(state->packet,DFALTPACKETSIZE); /*initial reasonable size*/
+
+    /* capture curl properties for this link from rc file1*/
+    stat = ocset_curlproperties(state);
+    if(stat != OC_NOERR) goto fail;
+
+    /* Set the one-time curl flags */
+    if((stat=ocset_flags_perlink(state))!= OC_NOERR) goto fail;
+#if 1 /* temporarily make per-link */
+    if((stat=ocset_flags_perfetch(state))!= OC_NOERR) goto fail;
+#endif
 
     if(statep) *statep = state;
-    return OCTHROW(stat);   
+    else {
+      if(state != NULL) ocfree(state);
+    }
+    return OCTHROW(stat);
 
 fail:
-    ocurifree(tmpurl);
+    ncurifree(tmpurl);
     if(state != NULL) ocfree(state);
     if(curl != NULL) occurlclose(curl);
     return OCTHROW(stat);
@@ -179,7 +202,7 @@ ocfetch(OCstate* state, const char* constraint, OCdxd kind, OCflags flags,
     OCtree* tree = NULL;
     OCnode* root = NULL;
     OCerror stat = OC_NOERR;
-    
+
     tree = (OCtree*)ocmalloc(sizeof(OCtree));
     MEMCHECK(tree,OC_ENOMEM);
     memset((void*)tree,0,sizeof(OCtree));
@@ -189,26 +212,25 @@ ocfetch(OCstate* state, const char* constraint, OCdxd kind, OCflags flags,
     if(tree->constraint == NULL)
 	tree->constraint = nulldup(constraint);
 
-    /* Set curl properties: pwd, flags, proxies, ssl */
-    if((stat=ocset_user_password(state))!= OC_NOERR) goto fail;
-    if((stat=ocset_curl_flags(state)) != OC_NOERR) goto fail;
-    if((stat=ocset_proxy(state)) != OC_NOERR) goto fail;
-    if((stat=ocset_ssl(state)) != OC_NOERR) goto fail;
+    /* Set per-fetch curl properties */
+#if 0 /* temporarily make per-link */
+    if((stat=ocset_flags_perfetch(state))!= OC_NOERR) goto fail;
+#endif
 
-    ocbytesclear(state->packet);
+    ncbytesclear(state->packet);
 
     switch (kind) {
     case OCDAS:
         stat = readDAS(state,tree);
 	if(stat == OC_NOERR) {
-            tree->text = ocbytesdup(state->packet);
+            tree->text = ncbytesdup(state->packet);
 	    if(tree->text == NULL) stat = OC_EDAS;
 	}
 	break;
     case OCDDS:
         stat = readDDS(state,tree);
 	if(stat == OC_NOERR) {
-            tree->text = ocbytesdup(state->packet);
+            tree->text = ncbytesdup(state->packet);
 	    if(tree->text == NULL) stat = OC_EDDS;
 	}
 	break;
@@ -237,13 +259,13 @@ ocfetch(OCstate* state, const char* constraint, OCdxd kind, OCflags flags,
     default:
 	break;
     }/*switch*/
+    /* Obtain any http code */
+    state->error.httpcode = ocfetchhttpcode(state->curl);
     if(stat != OC_NOERR) {
-	/* Obtain any http code */
-	state->error.httpcode = ocfetchhttpcode(state->curl);
 	if(state->error.httpcode >= 400) {
-	    oclog(OCLOGWARN,"oc_open: Could not read url; http error = %l",state->error.httpcode);
+	    nclog(NCLOGWARN,"oc_open: Could not read url; http error = %l",state->error.httpcode);
 	} else {
-	    oclog(OCLOGWARN,"oc_open: Could not read url");
+	    nclog(NCLOGWARN,"oc_open: Could not read url");
 	}
 	goto fail;
     }
@@ -252,8 +274,8 @@ ocfetch(OCstate* state, const char* constraint, OCdxd kind, OCflags flags,
     stat = DAPparse(state,tree,tree->text);
     /* Check and report on an error return from the server */
     if(stat == OC_EDAPSVC  && state->error.code != NULL) {
-	oclog(OCLOGERR,"oc_open: server error retrieving url: code=%s message=\"%s\"",
-		  state->error.code,	
+	nclog(NCLOGERR,"oc_open: server error retrieving url: code=%s message=\"%s\"",
+		  state->error.code,
 		  (state->error.message?state->error.message:""));
     }
     if(stat) {OCTHROWCHK(stat); goto fail;}
@@ -309,8 +331,8 @@ fprintf(stderr,"ocfetch.datadds.memory: datasize=%lu bod=%lu\n",
          */
 	if(dataError(tree->data.xdrs,state)) {
 	    stat = OC_EDATADDS;
-	    oclog(OCLOGERR,"oc_open: server error retrieving url: code=%s message=\"%s\"",
-		  state->error.code,	
+	    nclog(NCLOGERR,"oc_open: server error retrieving url: code=%s message=\"%s\"",
+		  state->error.code,
 		  (state->error.message?state->error.message:""));
 	    goto fail;
 	}
@@ -322,7 +344,7 @@ fprintf(stderr,"ocfetch.datadds.memory: datasize=%lu bod=%lu\n",
     }
 
     /* Put root into the state->trees list */
-    oclistpush(state->trees,(void*)root);
+    nclistpush(state->trees,(void*)root);
 
     if(rootp) *rootp = root;
     return stat;
@@ -338,66 +360,41 @@ fail:
 static OCerror
 createtempfile(OCstate* state, OCtree* tree)
 {
-    int fd = 0;
+    int stat = OC_NOERR;
+    char* path = NULL;
     char* name = NULL;
-    fd = createtempfile1(TMPPATH1,&name);
-    if(fd < 0)
-        fd = createtempfile1(TMPPATH2,&name);
-    if(fd < 0) {
-        oclog(OCLOGERR,"oc_open: attempt to open tmp file failed: %s",name);
-        if(name) free(name);
-	return errno;
-    }
+    int len;
+
+    len =
+	  strlen(ocglobalstate.tempdir)
+	  + 1 /* '/' */
+	  + strlen(DATADDSFILE);
+    path = (char*)malloc(len+1);
+    if(path == NULL) return OC_ENOMEM;
+    occopycat(path,len,3,ocglobalstate.tempdir,"/",DATADDSFILE);
+    stat = ocmktmp(path,&name);
+    free(path);
+    if(stat != OC_NOERR) goto fail;
 #ifdef OCDEBUG
-    oclog(OCLOGNOTE,"oc_open: using tmp file: %s",name);
+    nclog(NCLOGNOTE,"oc_open: creating tmp file: %s",name);
 #endif
     tree->data.filename = name; /* remember our tmp file name */
-    tree->data.file = fdopen(fd,"w+");
+    name = NULL;
+    tree->data.file = NCfopen(tree->data.filename,"w+");
     if(tree->data.file == NULL) return OC_EOPEN;
-    /* unlink the temp file so it will automatically be reclaimed */
-    if(ocdebug == 0) unlink(tree->data.filename);
-    return OC_NOERR;
-}
+    /* make the temp file so it will automatically be reclaimed on close */
+    if(ocdebug == 0)
+	ocremovefile(tree->data.filename);
+    return stat;
 
-int
-createtempfile1(char* tmppath, char** tmpnamep)
-{
-    int fd = 0;
-    char* tmpname = NULL;
-    size_t tmpsize = strlen(tmppath)+strlen("dataddsXXXXXX") + 1;
-    tmpname = (char*)malloc(tmpsize);
-    if(tmpname == NULL) return -1;
-    if(!occopycat(tmpname,tmpsize,1,tmppath))
-	return OC_EOVERRUN;
-#ifdef HAVE_MKSTEMP
-    if(!occoncat(tmpname,tmpsize,1,"dataddsXXXXXX"))
-	return OC_EOVERRUN;
-    /* Note Potential problem: old versions of this function
-       leave the file in mode 0666 instead of 0600 */
-    fd = mkstemp(tmpname);
-#else /* !HAVE_MKSTEMP */
-    /* Need to simulate by using some kind of pseudo-random number */
-    if(!occoncat(tmpname,tmpsize,1,"datadds"))
-	return OC_EOVERRUN;
-    {
-	int rno = rand();
-	char spid[7];
-	if(rno < 0) rno = -rno;
-        snprintf(spid,sizeof(spid),"%06d",rno);
-	if(!occoncat(tmpname,tmpsize,1,spid))
-	    return OC_EOVERRUN;
-#  ifdef _WIN32
-        fd=open(tmpname,O_RDWR|O_BINARY|O_CREAT|O_EXCL|FILE_ATTRIBUTE_TEMPORARY, _S_IREAD|_S_IWRITE);
-#  else
-        fd=open(tmpname,O_RDWR|O_CREAT|O_EXCL, S_IRWXU);
-#  endif
+fail:
+    if(name != NULL) {
+        nclog(NCLOGERR,"oc_open: attempt to create tmp file failed: %s",name);
+	free(name);
+    } else {
+        nclog(NCLOGERR,"oc_open: attempt to create tmp file failed: NULL");
     }
-#endif /* !HAVE_MKSTEMP */
-    if(tmpname == NULL) return -1;
-    if(tmpnamep) *tmpnamep = tmpname;
-    else
-      free(tmpname);
-    return fd;
+    return OCTHROW(stat);
 }
 
 void
@@ -408,26 +405,32 @@ occlose(OCstate* state)
 
     /* Warning: ocfreeroot will attempt to remove the root from state->trees */
     /* Ok in this case because we are popping the root out of state->trees */
-    for(i=0;i<oclistlength(state->trees);i++) {
-	OCnode* root = (OCnode*)oclistpop(state->trees);
+    for(i=0;i<nclistlength(state->trees);i++) {
+	OCnode* root = (OCnode*)nclistpop(state->trees);
 	ocroot_free(root);
     }
-    oclistfree(state->trees);
-    ocurifree(state->uri);
-    ocbytesfree(state->packet);
+    nclistfree(state->trees);
+    ncurifree(state->uri);
+    ncbytesfree(state->packet);
     ocfree(state->error.code);
     ocfree(state->error.message);
     ocfree(state->curlflags.useragent);
-    ocfree(state->curlflags.cookiejar);
-    ocfree(state->curlflags.cookiefile);
+    if(state->curlflags.cookiejar) {
+        if(state->curlflags.createdflags & COOKIECREATED)
+            ocremovefile(state->curlflags.cookiejar);
+	ocfree(state->curlflags.cookiejar);
+    }
+    ocfree(state->curlflags.netrc);
     ocfree(state->ssl.certificate);
     ocfree(state->ssl.key);
     ocfree(state->ssl.keypasswd);
     ocfree(state->ssl.cainfo);
-    ocfree(state->ssl.capath); 
+    ocfree(state->ssl.capath);
     ocfree(state->proxy.host);
-    ocfree(state->creds.username);
-    ocfree(state->creds.password);
+    ocfree(state->proxy.user);
+    ocfree(state->proxy.pwd);
+    ocfree(state->creds.user);
+    ocfree(state->creds.pwd);
     if(state->curl != NULL) occurlclose(state->curl);
     ocfree(state);
 }
@@ -448,12 +451,12 @@ ocextractddsinmemory(OCstate* state, OCtree* tree, OCflags flags)
     /* copy out the dds */
     if(ddslen > 0) {
         tree->text = (char*)ocmalloc(ddslen+1);
-        memcpy((void*)tree->text,(void*)ocbytescontents(state->packet),ddslen);
+        memcpy((void*)tree->text,(void*)ncbytescontents(state->packet),ddslen);
         tree->text[ddslen] = '\0';
     } else
 	tree->text = NULL;
     /* Extract the inmemory contents */
-    tree->data.memory = ocbytesextract(state->packet);
+    tree->data.memory = ncbytesextract(state->packet);
 #ifdef OCIGNORE
     /* guarantee the data part is on an 8 byte boundary */
     if(tree->data.bod % 8 != 0) {
@@ -475,7 +478,7 @@ ocextractddsinfile(OCstate* state, OCtree* tree, OCflags flags)
     size_t ddslen, bod, bodfound;
 
     /* Read until we find the separator (or EOF)*/
-    ocbytesclear(state->packet);
+    ncbytesclear(state->packet);
     rewind(tree->data.file);
     bodfound = 0;
     do {
@@ -484,7 +487,7 @@ ocextractddsinfile(OCstate* state, OCtree* tree, OCflags flags)
 	/* read chunks of the file until we find the separator*/
         count = fread(chunk,1,sizeof(chunk),tree->data.file);
 	if(count <= 0) break; /* EOF;*/
-        ocbytesappendn(state->packet,chunk,count);
+        ncbytesappendn(state->packet,chunk,count);
 	bodfound = ocfindbod(state->packet,&bod,&ddslen);
     } while(!bodfound);
     if(!bodfound) {/* No BOD; pretend */
@@ -500,13 +503,14 @@ fprintf(stderr,"missing bod: ddslen=%lu bod=%lu\n",
     /* copy out the dds */
     if(ddslen > 0) {
         tree->text = (char*)ocmalloc(ddslen+1);
-        memcpy((void*)tree->text,(void*)ocbytescontents(state->packet),ddslen);
+        memcpy((void*)tree->text,(void*)ncbytescontents(state->packet),ddslen);
         tree->text[ddslen] = '\0';
     } else
 	tree->text = NULL;
     /* reset the position of the tmp file*/
-    fseek(tree->data.file,tree->data.bod,SEEK_SET);
-    if(tree->text == NULL) stat = OC_EDATADDS;
+    if(fseek(tree->data.file,(long)tree->data.bod,SEEK_SET) < 0
+       || tree->text == NULL)
+	stat = OC_EDATADDS;
     return OCTHROW(stat);
 }
 
@@ -550,60 +554,112 @@ ocupdatelastmodifieddata(OCstate* state)
     OCerror status = OC_NOERR;
     long lastmodified;
     char* base = NULL;
-    base = ocuribuild(state->uri,NULL,NULL,OCURIENCODE);
+    base = ncuribuild(state->uri,NULL,NULL,NCURIENCODE);
     status = ocfetchlastmodified(state->curl, base, &lastmodified);
     free(base);
     if(status == OC_NOERR) {
 	state->datalastmodified = lastmodified;
     }
-    return status;
+    return OCTHROW(status);
 }
 
 /*
-    Set curl properties for link based on rc files
+    Set curl properties for link based on rc files etc.
 */
-static void
-ocsetcurlproperties(OCstate* state)
+static OCerror
+ocset_curlproperties(OCstate* state)
 {
-    CURLcode cstat = CURLE_OK;
+    OCerror stat = OC_NOERR;
+
+    /* extract the relevant triples int state */
+    ocrc_process(state);
 
-    /* process the triple store wrt to this state */
-    if(ocdodsrc_process(state) != OC_NOERR) {
-	oclog(OCLOGERR,"Malformed .opendaprc configuration file");
-	goto fail;
-    }
-    if(state->creds.username == NULL && state->creds.password == NULL) {
-        if(state->uri->user != NULL && state->uri->password != NULL) {
-	    /* this overrides .dodsrc */
-            if(state->creds.password) free(state->creds.password);
-            state->creds.password = nulldup(state->uri->password);
-            if(state->creds.username) free(state->creds.username);
-            state->creds.username = nulldup(state->uri->user);
-	}
-    }
     if(state->curlflags.useragent == NULL) {
         size_t len = strlen(DFALTUSERAGENT) + strlen(VERSION) + 1;
 	char* agent = (char*)malloc(len+1);
-    if(occopycat(agent,len,2,DFALTUSERAGENT,VERSION))
+	if(occopycat(agent,len,2,DFALTUSERAGENT,VERSION))
 	    state->curlflags.useragent = agent;
+	else
+	    free(agent);
     }
-    return;
+
+    /* Some servers (e.g. thredds and columbia) appear to require a place
+       to put cookies in order for some security functions to work
+    */
+    if(state->curlflags.cookiejar != NULL
+       && strlen(state->curlflags.cookiejar) == 0) {
+	free(state->curlflags.cookiejar);
+	state->curlflags.cookiejar = NULL;
+    }
+
+    if(state->curlflags.cookiejar == NULL) {
+	/* If no cookie file was defined, define a default */
+        int stat;
+        char* path = NULL;
+        char* name = NULL;
+        int len;
+	errno = 0;
+	/* Create the unique cookie file name */
+        len =
+	  strlen(ocglobalstate.tempdir)
+	  + 1 /* '/' */
+	  + strlen("occookies");
+        path = (char*)malloc(len+1);
+        if(path == NULL) return OC_ENOMEM;
+        occopycat(path,len,3,ocglobalstate.tempdir,"/","occookies");
+        stat = ocmktmp(path,&name);
+        free(path);
+	state->curlflags.cookiejar = name;
+	state->curlflags.createdflags |= COOKIECREATED;
+	if(stat != OC_NOERR && errno != EEXIST) {
+	    fprintf(stderr,"Cannot create cookie file\n");
+	    goto fail;
+	}
+	errno = 0;
+    }
+    OCASSERT(state->curlflags.cookiejar != NULL);
+
+    /* Make sure the cookie jar exists and can be read and written */
+    {
+	FILE* f = NULL;
+	char* fname = state->curlflags.cookiejar;
+	/* See if the file exists already */
+        f = NCfopen(fname,"r");
+	if(f == NULL) {
+	    /* Ok, create it */
+	    f = NCfopen(fname,"w+");
+	    if(f == NULL) {
+	        fprintf(stderr,"Cookie file cannot be read and written: %s\n",fname);
+	        {stat = OC_EPERM; goto fail;}
+	    }
+	} else { /* test if file can be written */
+	    fclose(f);
+	    f = NCfopen(fname,"r+");
+	    if(f == NULL) {
+	        fprintf(stderr,"Cookie file is cannot be written: %s\n",fname);
+	        {stat = OC_EPERM; goto fail;}
+	    }
+	}
+	if(f != NULL) fclose(f);
+    }
+
+#if 0
+    /* Create a netrc file if specified  and required,
+       where required => >1 NETRC triples exist */
+    if(ocrc_netrc_required(state)) {
+	/* WARNING: it appears that a user+pwd was specified specifically, then
+           the netrc file will be completely disabled. */
+	if(state->creds.userpwd != NULL) {
+  	    nclog(NCLOGWARN,"The rc file specifies both netrc and user+pwd; this will cause curl to ignore the netrc file");
+	}
+	stat = oc_build_netrc(state);
+    }
+#endif
+
+    return stat;
 
 fail:
-    if(cstat != CURLE_OK)
-	oclog(OCLOGERR, "curl error: %s", curl_easy_strerror(cstat));
-    return;
-}
-
-OCerror
-ocsetuseragent(OCstate* state, const char* agent)
-{
-    if(state->curlflags.useragent != NULL)
-	free(state->curlflags.useragent);
-    state->curlflags.useragent = strdup(agent);
-    if(state->curlflags.useragent == NULL)
-	return OC_ENOMEM;
-    return OC_NOERR;
+    return OCTHROW(stat);
 }
 
 static char* ERROR_TAG = "Error ";
@@ -613,7 +669,8 @@ dataError(XXDR* xdrs, OCstate* state)
 {
     int depth=0;
     int errfound = 0;
-    off_t ckp=0,avail=0,i=0;
+    off_t ckp=0,avail=0;
+    int i=0;
     char* errmsg = NULL;
     char errortext[16]; /* bigger thant |ERROR_TAG|*/
     avail = xxdr_getavail(xdrs);
@@ -628,17 +685,17 @@ dataError(XXDR* xdrs, OCstate* state)
     /* Try to locate the whole error body */
     xxdr_setpos(xdrs,ckp);
     for(depth=0,i=0;i<avail;i++) {
-	xxdr_getbytes(xdrs,errortext,1);
+	xxdr_getbytes(xdrs,errortext,(off_t)1);
 	if(errortext[0] == CLBRACE) depth++;
 	else if(errortext[0] == CRBRACE) {
 	    depth--;
 	    if(depth == 0) {i++; break;}
 	}
-    }    
-    errmsg = (char*)malloc(i+1);
+    }
+    errmsg = (char*)malloc((size_t)i+1);
     if(errmsg == NULL) {errfound = 1; goto done;}
     xxdr_setpos(xdrs,ckp);
-    xxdr_getbytes(xdrs,errmsg,i);
+    xxdr_getbytes(xdrs,errmsg,(off_t)i);
     errmsg[i] = '\0';
     state->error.message = errmsg;
     state->error.code = strdup("?");
@@ -649,3 +706,47 @@ done:
     xxdr_setpos(xdrs,ckp);
     return errfound;
 }
+
+/**************************************************/
+/* Curl option functions */
+/*
+Note that if we set the option in curlflags,
+then we need to also invoke the ocset_curlopt
+to update the curl flags in libcurl.
+*/
+
+OCerror
+ocset_useragent(OCstate* state, const char* agent)
+{
+    OCerror stat = OC_NOERR;
+    if(state->curlflags.useragent != NULL)
+	free(state->curlflags.useragent);
+    state->curlflags.useragent = strdup(agent);
+    if(state->curlflags.useragent == NULL)
+	return OCTHROW(OC_ENOMEM);
+    stat = ocset_curlflag(state,CURLOPT_USERAGENT);
+    return stat;
+}
+
+OCerror
+ocset_netrc(OCstate* state, const char* path)
+{
+    OCerror stat = OC_NOERR;
+    if(state->curlflags.netrc != NULL)
+	free(state->curlflags.netrc);
+    state->curlflags.netrc = strdup(path);
+    if(state->curlflags.netrc == NULL)
+	return OCTHROW(OC_ENOMEM);
+    stat = ocset_curlflag(state,CURLOPT_NETRC);
+    return stat;
+}
+
+static void
+ocremovefile(const char* path)
+{
+#ifdef _MSC_VER
+    DeleteFile(path);
+#else
+    remove(path);
+#endif
+}
