@@ -9,9 +9,11 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "netcdf.h"
 #include "ocinternal.h"
 #include "ocdebug.h"
-#include "oclog.h"
+#include "nclog.h"
+#include "ncwinpath.h"
 
 #define OCRCFILEENV "DAPRCFILE"
 
@@ -24,7 +26,6 @@ static OCerror rc_search(const char* prefix, const char* rcfile, char** pathp);
 
 static int rcreadline(FILE* f, char* more, int morelen);
 static void rctrim(char* text);
-static char* combinecredentials(const char* user, const char* pwd);
 
 static void storedump(char* msg, struct OCTriple*, int ntriples);
 
@@ -46,23 +47,55 @@ occredentials_in_url(const char *url)
     return 0;
 }
 
+/*
+Given form user:pwd, parse into user and pwd
+and do %xx unescaping
+*/
 static OCerror
-ocextract_credentials(const char *url, char **userpwd, char **result_url)
+parsecredentials(const char* userpwd, char** userp, char** pwdp)
 {
-    OCURI* parsed = NULL;
-    if(!ocuriparse(url,&parsed))
+    char* user = NULL;
+    char* pwd = NULL;
+
+    if(userpwd == NULL)
+	return OC_EINVAL;
+    user = strdup(userpwd);
+    if(user == NULL)
+	return NC_ENOMEM;
+    pwd = strchr(user,':');
+    if(pwd == NULL)
+	return OC_EINVAL;
+    *pwd = '\0';
+    pwd++;
+    if(userp)
+	*userp = ncuridecode(user);
+    if(pwdp)
+	*pwdp = ncuridecode(pwd);
+    free(user);
+    return OC_NOERR;
+}
+
+static OCerror
+ocextract_credentials(const char *url, char **user, char** pwd, char **result_url)
+{
+    NCURI* parsed = NULL;
+
+    if(url == NULL || ncuriparse(url,&parsed) != NCU_OK)
 	return OCTHROW(OC_EBADURL);
-    if(parsed->userpwd == NULL) {
-	ocurifree(parsed);
+    if(parsed->user != NULL || parsed->password == NULL) {
+	ncurifree(parsed);
 	return OCTHROW(OC_EBADURL);
     }
-    if(userpwd) *userpwd = strdup(parsed->userpwd);
-    ocurifree(parsed);
+    if(user)
+	*user = parsed->user;
+    if(pwd)
+	*pwd = parsed->password;
+    ncurifree(parsed);
     return OC_NOERR;
 }
 
 char*
-occombinehostport(const OCURI* uri)
+occombinehostport(const NCURI* uri)
 {
     char* hp;
     int len = 0;
@@ -83,24 +116,40 @@ occombinehostport(const OCURI* uri)
     return hp;
 }
 
+#if 0
+/*
+Combine user and pwd into the user:pwd form.
+Note that we must %xx escape the user and the pwd
+*/
 static char*
 combinecredentials(const char* user, const char* pwd)
 {
     int userPassSize;
     char *userPassword;
+    char *escapeduser = NULL;
+    char* escapedpwd = NULL;
 
-    if(user == NULL) user = "";
-    if(pwd == NULL) pwd = "";
-
-    userPassSize = strlen(user) + strlen(pwd) + 2;
+    if(user == NULL || pwd == NULL)
+	return NULL;	
+ 
+    userPassSize = 3*strlen(user) + 3*strlen(pwd) + 2; /* times 3 for escapes */
     userPassword = malloc(sizeof(char) * userPassSize);
     if (!userPassword) {
-        oclog(OCLOGERR,"Out of Memory\n");
+        nclog(NCLOGERR,"Out of Memory\n");
 	return NULL;
     }
-    occopycat(userPassword,userPassSize-1,3,user,":",pwd);
+    escapeduser = ncuriencodeuserpwd(user);
+    escapedpwd = ncuriencodeuserpwd(pwd);
+    if(escapeduser == NULL || escapedpwd == NULL) {
+        nclog(NCLOGERR,"Out of Memory\n");
+	return NULL;
+    }
+    occopycat(userPassword,userPassSize-1,3,escapeduser,":",escapedpwd);
+    free(escapeduser);
+    free(escapedpwd);
     return userPassword;
 }
+#endif
 
 static int
 rcreadline(FILE* f, char* more, int morelen)
@@ -156,7 +205,7 @@ ocparseproxy(OCstate* state, char* v)
 	return OC_NOERR; /* nothing there*/
     if (occredentials_in_url(v)) {
         char *result_url = NULL;
-        ocextract_credentials(v, &state->proxy.userpwd, &result_url);
+        ocextract_credentials(v, &state->proxy.user, &state->proxy.pwd, &result_url);
         v = result_url;
     }
     /* allocating a bit more than likely needed ... */
@@ -204,11 +253,11 @@ ocparseproxy(OCstate* state, char* v)
      state->proxy.password[p_len] = '\0';
 #endif /*0*/
      if (ocdebug > 1) {
-         oclog(OCLOGNOTE,"host name: %s", state->proxy.host);
+         nclog(NCLOGNOTE,"host name: %s", state->proxy.host);
 #ifdef INSECURE
-         oclog(OCLOGNOTE,"user+pwd: %s", state->proxy.userpwd);
+         nclog(NCLOGNOTE,"user+pwd: %s+%s", state->proxy.user,state->proxy.pwd);
 #endif
-         oclog(OCLOGNOTE,"port number: %d", state->proxy.port);
+         nclog(NCLOGNOTE,"port number: %d", state->proxy.port);
     }
     if(v) free(v);
     return OC_NOERR;
@@ -228,7 +277,7 @@ sorttriplestore(struct OCTriplestore* store)
 
     sorted = (struct OCTriple*)malloc(sizeof(struct OCTriple)*store->ntriples);
     if(sorted == NULL) {
-        oclog(OCLOGERR,"sorttriplestore: out of memory");
+        nclog(NCLOGERR,"sorttriplestore: out of memory");
         return;
     }
 
@@ -278,9 +327,9 @@ ocrc_compile(const char* path)
 
     ocrc->ntriples = 0; /* reset; nothing to free */
 
-    in_file = fopen(path, "r"); /* Open the file to read it */
+    in_file = NCfopen(path, "r"); /* Open the file to read it */
     if (in_file == NULL) {
-        oclog(OCLOGERR, "Could not open configuration file: %s",path);
+        nclog(NCLOGERR, "Could not open configuration file: %s",path);
         return OC_EPERM;
     }
 
@@ -290,7 +339,7 @@ ocrc_compile(const char* path)
         if(!rcreadline(in_file,line0,sizeof(line0))) break;
         linecount++;
         if(linecount >= MAXRCLINES) {
-            oclog(OCLOGERR, ".rc has too many lines");
+            nclog(NCLOGERR, ".rc has too many lines");
             return 0;
         }
         line = line0;
@@ -300,7 +349,7 @@ ocrc_compile(const char* path)
         rctrim(line);  /* trim leading and trailing blanks */
 	if(strlen(line) == 0) continue;
         if(strlen(line) >= MAXRCLINESIZE) {
-            oclog(OCLOGERR, "%s line too long: %s",path,line0);
+            nclog(NCLOGERR, "%s line too long: %s",path,line0);
             continue; /* ignore it */
         }
         /* setup */
@@ -308,18 +357,18 @@ ocrc_compile(const char* path)
         ocrc->triples[ocrc->ntriples].key[0] = '\0';
         ocrc->triples[ocrc->ntriples].value[0] = '\0';
         if(line[0] == LTAG) {
-	    OCURI* uri;
+	    NCURI* uri;
             char* url = ++line;
             char* rtag = strchr(line,RTAG);
             if(rtag == NULL) {
-                oclog(OCLOGERR, "Malformed [url] in %s entry: %s",path,line);
+                nclog(NCLOGERR, "Malformed [url] in %s entry: %s",path,line);
                 continue;
             }
             line = rtag + 1;
             *rtag = '\0';
             /* compile the url and pull out the host */
-	    if(!ocuriparse(url,&uri)) {
-                oclog(OCLOGERR, "Malformed [url] in %s entry: %s",path,line);
+	    if(ncuriparse(url,&uri) != NCU_OK) {
+                nclog(NCLOGERR, "Malformed [url] in %s entry: %s",path,line);
 		continue;
 	    }
             strncpy(ocrc->triples[ocrc->ntriples].host,uri->host,MAXRCLINESIZE-1);
@@ -327,7 +376,7 @@ ocrc_compile(const char* path)
                 strncat(ocrc->triples[ocrc->ntriples].host,":",MAXRCLINESIZE-1);
                 strncat(ocrc->triples[ocrc->ntriples].host,uri->port,MAXRCLINESIZE-1);
 	    }
-	    ocurifree(uri);
+	    ncurifree(uri);
         }
         /* split off key and value */
         key=line;
@@ -363,7 +412,7 @@ ocrc_load(void)
     char* path = NULL;
 
     if(ocglobalstate.rc.ignore) {
-        oclog(OCLOGDBG,"No runtime configuration file specified; continuing");
+        nclog(NCLOGDBG,"No runtime configuration file specified; continuing");
 	return OC_NOERR;
     }
     if(ocglobalstate.rc.loaded) return OC_NOERR;
@@ -371,8 +420,8 @@ ocrc_load(void)
     /* locate the configuration files in the following order:
        1. specified by set_rcfile
        2. set by DAPRCFILE env variable
-       3. '.'
-       4. $HOME
+       3. '.'/<rcfile>
+       4. $HOME/<rcfile>
     */
     if(ocglobalstate.rc.rcfile != NULL) { /* always use this */
 	path = strdup(ocglobalstate.rc.rcfile);
@@ -392,12 +441,12 @@ ocrc_load(void)
 	}
     }
     if(path == NULL) {
-        oclog(OCLOGDBG,"Cannot find runtime configuration file; continuing");
+        nclog(NCLOGDBG,"Cannot find runtime configuration file; continuing");
     } else {
 	if(ocdebug > 0)
 	    fprintf(stderr, "RC file: %s\n", path);
         if(ocrc_compile(path) == 0) {
-	    oclog(OCLOGERR, "Error parsing %s\n",path);
+	    nclog(NCLOGERR, "Error parsing %s\n",path);
 	    stat = OC_ERCFILE;
 	}
     }
@@ -413,8 +462,7 @@ ocrc_process(OCstate* state)
 {
     OCerror stat = OC_NOERR;
     char* value = NULL;
-    OCURI* uri = state->uri;
-    char* url_userpwd = NULL;
+    NCURI* uri = state->uri;
     char* url_hostport = NULL;
 
     if(!ocglobalstate.initialized)
@@ -426,7 +474,6 @@ ocrc_process(OCstate* state)
        to getinfo e.g. user:pwd from url
     */
 
-    url_userpwd = uri->userpwd;
     url_hostport = occombinehostport(uri);
     if(url_hostport == NULL)
 	return OC_ENOMEM;
@@ -435,23 +482,23 @@ ocrc_process(OCstate* state)
     if(value != NULL) {
         if(atoi(value)) state->curlflags.compress = 1;
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.DEFLATE: %ld", state->curlflags.compress);
+            nclog(NCLOGNOTE,"HTTP.DEFLATE: %ld", state->curlflags.compress);
     }
     if((value = ocrc_lookup("HTTP.VERBOSE",url_hostport)) != NULL) {
         if(atoi(value)) state->curlflags.verbose = 1;
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.VERBOSE: %ld", state->curlflags.verbose);
+            nclog(NCLOGNOTE,"HTTP.VERBOSE: %ld", state->curlflags.verbose);
     }
     if((value = ocrc_lookup("HTTP.TIMEOUT",url_hostport)) != NULL) {
         if(atoi(value)) state->curlflags.timeout = atoi(value);
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.TIMEOUT: %ld", state->curlflags.timeout);
+            nclog(NCLOGNOTE,"HTTP.TIMEOUT: %ld", state->curlflags.timeout);
     }
     if((value = ocrc_lookup("HTTP.USERAGENT",url_hostport)) != NULL) {
         if(atoi(value)) state->curlflags.useragent = strdup(value);
         if(state->curlflags.useragent == NULL) {stat = OC_ENOMEM; goto done;}
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.USERAGENT: %s", state->curlflags.useragent);
+            nclog(NCLOGNOTE,"HTTP.USERAGENT: %s", state->curlflags.useragent);
     }
 
     if(
@@ -463,14 +510,14 @@ ocrc_process(OCstate* state)
         state->curlflags.cookiejar = strdup(value);
         if(state->curlflags.cookiejar == NULL) {stat = OC_ENOMEM; goto done;}
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.COOKIEJAR: %s", state->curlflags.cookiejar);
+            nclog(NCLOGNOTE,"HTTP.COOKIEJAR: %s", state->curlflags.cookiejar);
     }
 
     if((value = ocrc_lookup("HTTP.PROXY_SERVER",url_hostport)) != NULL) {
         stat = ocparseproxy(state,value);
         if(stat != OC_NOERR) goto done;
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.PROXY_SERVER: %s", value);
+            nclog(NCLOGNOTE,"HTTP.PROXY_SERVER: %s", value);
     }
 
     if((value = ocrc_lookup("HTTP.SSL.VALIDATE",url_hostport)) != NULL) {
@@ -478,7 +525,7 @@ ocrc_process(OCstate* state)
 	    state->ssl.verifypeer = 1;
 	    state->ssl.verifyhost = 1;
             if(ocdebug > 0)
-                oclog(OCLOGNOTE,"HTTP.SSL.VALIDATE: %ld", 1);
+                nclog(NCLOGNOTE,"HTTP.SSL.VALIDATE: %ld", 1);
 	}
     }
 
@@ -486,35 +533,35 @@ ocrc_process(OCstate* state)
         state->ssl.certificate = strdup(value);
         if(state->ssl.certificate == NULL) {stat = OC_ENOMEM; goto done;}
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.SSL.CERTIFICATE: %s", state->ssl.certificate);
+            nclog(NCLOGNOTE,"HTTP.SSL.CERTIFICATE: %s", state->ssl.certificate);
     }
 
     if((value = ocrc_lookup("HTTP.SSL.KEY",url_hostport)) != NULL) {
         state->ssl.key = strdup(value);
         if(state->ssl.key == NULL) {stat = OC_ENOMEM; goto done;}
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.SSL.KEY: %s", state->ssl.key);
+            nclog(NCLOGNOTE,"HTTP.SSL.KEY: %s", state->ssl.key);
     }
 
     if((value = ocrc_lookup("HTTP.SSL.KEYPASSWORD",url_hostport)) != NULL) {
         state->ssl.keypasswd = strdup(value);
         if(state->ssl.keypasswd == NULL) {stat = OC_ENOMEM; goto done;}
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.SSL.KEYPASSWORD: %s", state->ssl.keypasswd);
+            nclog(NCLOGNOTE,"HTTP.SSL.KEYPASSWORD: %s", state->ssl.keypasswd);
     }
 
     if((value = ocrc_lookup("HTTP.SSL.CAINFO",url_hostport)) != NULL) {
         state->ssl.cainfo = strdup(value);
         if(state->ssl.cainfo == NULL) {stat = OC_ENOMEM; goto done;}
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.SSL.CAINFO: %s", state->ssl.cainfo);
+            nclog(NCLOGNOTE,"HTTP.SSL.CAINFO: %s", state->ssl.cainfo);
     }
 
     if((value = ocrc_lookup("HTTP.SSL.CAPATH",url_hostport)) != NULL) {
         state->ssl.capath = strdup(value);
         if(state->ssl.capath == NULL) {stat = OC_ENOMEM; goto done;}
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.SSL.CAPATH: %s", state->ssl.capath);
+            nclog(NCLOGNOTE,"HTTP.SSL.CAPATH: %s", state->ssl.capath);
     }
 
     if((value = ocrc_lookup("HTTP.SSL.VERIFYPEER",url_hostport)) != NULL) {
@@ -528,7 +575,7 @@ ocrc_process(OCstate* state)
             tf = 1; /* default if not null */
         state->ssl.verifypeer = tf;
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.SSL.VERIFYPEER: %d", state->ssl.verifypeer);
+            nclog(NCLOGNOTE,"HTTP.SSL.VERIFYPEER: %d", state->ssl.verifypeer);
 	free(s);
     }
 
@@ -538,26 +585,33 @@ ocrc_process(OCstate* state)
         state->curlflags.netrc = strdup(value);
         if(state->curlflags.netrc == NULL) {stat = OC_ENOMEM; goto done;}
         if(ocdebug > 0)
-            oclog(OCLOGNOTE,"HTTP.NETRC: %s", state->curlflags.netrc);
+            nclog(NCLOGNOTE,"HTTP.NETRC: %s", state->curlflags.netrc);
     }
 
     { /* Handle various cases for user + password */
 	/* First, see if the user+pwd was in the original url */
-	char* userpwd = NULL;
 	char* user = NULL;
 	char* pwd = NULL;
-	if(url_userpwd != NULL)
-	    userpwd = url_userpwd;
-	else {
+	if(uri->user != NULL && uri->password != NULL) {
+	    user = uri->user;
+	    pwd = uri->password;
+	} else {
    	    user = ocrc_lookup("HTTP.CREDENTIALS.USER",url_hostport);
 	    pwd = ocrc_lookup("HTTP.CREDENTIALS.PASSWORD",url_hostport);
-	    userpwd = ocrc_lookup("HTTP.CREDENTIALS.USERPASSWORD",url_hostport);
 	}
-	if(userpwd == NULL && user != NULL && pwd != NULL) {
-	    userpwd = combinecredentials(user,pwd);
-	    state->creds.userpwd = userpwd;
-	} else if(userpwd != NULL)
-	    state->creds.userpwd = strdup(userpwd);
+	if(user != NULL && pwd != NULL) {
+            state->creds.user = strdup(user);
+            state->creds.pwd = strdup(pwd);
+	} else {
+	    /* Could not get user and pwd, so try USERPASSWORD */
+	    const char* userpwd = ocrc_lookup("HTTP.CREDENTIALS.USERPASSWORD",url_hostport);
+	    if(userpwd != NULL) {
+		stat = parsecredentials(userpwd,&user,&pwd);
+		if(stat) goto done;
+                state->creds.user = user;
+                state->creds.pwd = pwd;
+	    }
+        }
     }
 
 done:
@@ -622,7 +676,7 @@ storedump(char* msg, struct OCTriple* triples, int ntriples)
     if(ntriples < 0 ) ntriples= ocrc->ntriples;
     for(i=0;i<ntriples;i++) {
         fprintf(stderr,"\t%s\t%s\t%s\n",
-                (strlen(triples[i].host)==0?"--":triples[i].host),
+                (triples[i].host == NULL || strlen(triples[i].host)==0?"--":triples[i].host),
                 triples[i].key,
                 triples[i].value);
     }
@@ -673,12 +727,12 @@ ocreadrc(void)
 	}
     }
     if(path == NULL) {
-        oclog(OCLOGDBG,"Cannot find runtime configuration file; continuing");
+        nclog(NCLOGDBG,"Cannot find runtime configuration file; continuing");
     } else {
 	if(ocdebug > 0)
 	    fprintf(stderr, "DODS RC file: %s\n", path);
         if(ocdodsrc_read(path) == 0) {
-	    oclog(OCLOGERR, "Error parsing %s\n",path);
+	    nclog(NCLOGERR, "Error parsing %s\n",path);
 	    stat = OC_ERCFILE;
 	}
     }
@@ -713,9 +767,7 @@ rc_search(const char* prefix, const char* rcname, char** pathp)
 	goto done;
     }
     /* see if file is readable */
-    f = fopen(path,"r");
-    if(f != NULL)
-        oclog(OCLOGDBG, "Found rc file=%s",path);
+    f = NCfopen(path,"r");
 done:
     if(f == NULL || stat != OC_NOERR) {
       if(path != NULL)
